@@ -3,8 +3,8 @@
 
   rbgdk-pixbuf-format.c -
 
-  $Author: mutoh $
-  $Date: 2005/03/05 17:01:36 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:28 $
 
   Copyright (C) 2004 Masao Mutoh
 ************************************************/
@@ -119,8 +119,8 @@ get_signature(self)
     VALUE array = rb_ary_new();
     int i = 0;
     while(signature[i].prefix){
-        rb_ary_push(array, rb_ary_new3(3, CSTR2RVAL(signature[i].prefix),
-                                       CSTR2RVAL(signature[i].mask),
+        rb_ary_push(array, rb_ary_new3(3, CSTR2RVAL((const char*)signature[i].prefix),
+                                       CSTR2RVAL((const char*)signature[i].mask),
                                        INT2NUM(signature[i].relevance)));
         i++;
     }
@@ -145,7 +145,7 @@ static VALUE
 set_disabled(self, disabled)
     VALUE self, disabled;
 {
-    gdk_pixbuf_format_set_disabled(_SELF(self), RTEST(disabled));
+    gdk_pixbuf_format_set_disabled(_SELF(self), RVAL2CBOOL(disabled));
     return self;
 }
 static VALUE
