@@ -3,8 +3,8 @@
 
   rbgdk-pixbuf.c -
 
-  $Author: mutoh $
-  $Date: 2006/12/27 17:49:45 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:28 $
 
   Copyright (C) 2002-2004 Masao Mutoh
   Copyright (C) 2000 Yasushi Shoji
@@ -128,14 +128,14 @@ initialize(argc, argv, self)
     if (argc == 7){
         buf = gdk_pixbuf_new_from_data((const guchar*)RVAL2CSTR(arg1), 
                                        RVAL2GENUM(arg2, GDK_TYPE_COLORSPACE),
-                                       RTEST(arg3),   NUM2INT(arg4),
+                                       RVAL2CBOOL(arg3),   NUM2INT(arg4),
                                        NUM2INT(arg5), NUM2INT(arg6),
                                        NUM2INT(arg7), NULL, NULL);
         if (buf == NULL){
             rb_gc();
             buf = gdk_pixbuf_new_from_data((const guchar*)RVAL2CSTR(arg1), 
                                            RVAL2GENUM(arg2, GDK_TYPE_COLORSPACE),
-                                           RTEST(arg3),   NUM2INT(arg4),
+                                           RVAL2CBOOL(arg3),   NUM2INT(arg4),
                                            NUM2INT(arg5), NUM2INT(arg6),
                                            NUM2INT(arg7), NULL, NULL);
             if (buf == NULL) NOMEM_ERROR(&error);
@@ -156,12 +156,12 @@ initialize(argc, argv, self)
             }
         } else if (rb_obj_is_kind_of(arg1, GTYPE2CLASS(GDK_TYPE_COLORSPACE))){
             buf = gdk_pixbuf_new(RVAL2GENUM(arg1, GDK_TYPE_COLORSPACE),
-                                 RTEST(arg2), NUM2INT(arg3),
+                                 RVAL2CBOOL(arg2), NUM2INT(arg3),
                                  NUM2INT(arg4), NUM2INT(arg5));
             if (buf == NULL){
                 rb_gc();
                 buf = gdk_pixbuf_new(RVAL2GENUM(arg1, GDK_TYPE_COLORSPACE),
-                                     RTEST(arg2), NUM2INT(arg3),
+                                     RVAL2CBOOL(arg2), NUM2INT(arg3),
                                      NUM2INT(arg4), NUM2INT(arg5));
                 if (buf == NULL) NOMEM_ERROR(&error);
             }
@@ -170,18 +170,24 @@ initialize(argc, argv, self)
         }
     } else if (argc == 4) {
 #if RBGDK_PIXBUF_CHECK_VERSION(2,6,0)
+        int width = NUM2INT(arg2);
+        int height = NUM2INT(arg3);
+#if ! RBGDK_PIXBUF_CHECK_VERSION(2,8,0)
+        if (width < 0 || height < 0)
+            rb_warning("For scaling on load, a negative value for width or height are not supported in GTK+ < 2.8.0");
+#endif
         buf = gdk_pixbuf_new_from_file_at_scale(RVAL2CSTR(arg1),
-                                                NUM2INT(arg2), NUM2INT(arg3), 
-                                                RTEST(arg4), &error);
+                                                width, height,
+                                                RVAL2CBOOL(arg4), &error);
         if (buf == NULL){
             rb_gc();
             error = NULL;
             buf = gdk_pixbuf_new_from_file_at_scale(RVAL2CSTR(arg1),
                                                     NUM2INT(arg2), NUM2INT(arg3), 
-                                                    RTEST(arg4), &error);
+                                                    RVAL2CBOOL(arg4), &error);
         }
 #else
-        rb_warning("Not supported in GTK+-2.4.x.");
+        rb_warning("Scaling on load not supported in GTK+ < 2.6.0");
         buf = gdk_pixbuf_new_from_file(RVAL2CSTR(arg1), &error);
         if (buf == NULL){
             error = NULL;
@@ -200,7 +206,7 @@ initialize(argc, argv, self)
                                                    NUM2INT(arg2), NUM2INT(arg3), &error);
         }
 #else
-        rb_warning("Not supported in GTK+-2.0.x.");
+        rb_warning("Sizing on load not supported in GTK+ < 2.4.0");
         buf = gdk_pixbuf_new_from_file(RVAL2CSTR(arg1), &error);
         if (buf == NULL){
             error = NULL;
@@ -210,16 +216,16 @@ initialize(argc, argv, self)
 #endif
     } else if (argc == 2){
         int i;
-        int len = RARRAY(arg1)->len; 
+        int len = RARRAY_LEN(arg1); 
         guint8 *gstream = g_new(guint8, len);
         for (i = 0; i < len; i++){
-            gstream[i] = (guint8)NUM2UINT(RARRAY(arg1)->ptr[i]);
+            gstream[i] = (guint8)NUM2UINT(RARRAY_PTR(arg1)[i]);
         }      
-        buf = gdk_pixbuf_new_from_inline(len, gstream, RTEST(arg2), &error);
+        buf = gdk_pixbuf_new_from_inline(len, gstream, RVAL2CBOOL(arg2), &error);
         if (buf == NULL){
             rb_gc();
             error = NULL;
-            buf = gdk_pixbuf_new_from_inline(len, gstream, RTEST(arg2), &error);
+            buf = gdk_pixbuf_new_from_inline(len, gstream, RVAL2CBOOL(arg2), &error);
         }
         /* need to manage the returned value */
         rb_ivar_set(self, id_pixdata, Data_Wrap_Struct(rb_cData, NULL, g_free, gstream));
@@ -233,9 +239,9 @@ initialize(argc, argv, self)
             }
         } else if (TYPE(arg1) == T_ARRAY) {
             int i;
-            gchar** data = ALLOCA_N(gchar*, RARRAY(arg1)->len);
-            for (i=0; i < RARRAY(arg1)->len; i++) {
-		data[i] = RVAL2CSTR(RARRAY(arg1)->ptr[i]);
+            gchar** data = ALLOCA_N(gchar*, RARRAY_LEN(arg1));
+            for (i=0; i < RARRAY_LEN(arg1); i++) {
+		data[i] = RVAL2CSTR(RARRAY_PTR(arg1)[i]);
             }
             buf = gdk_pixbuf_new_from_xpm_data((const gchar**)data);
             if (buf == NULL){
@@ -260,7 +266,13 @@ static VALUE
 copy(self)
     VALUE self;
 {
-    return GOBJ2RVAL(gdk_pixbuf_copy(_SELF(self)));
+    VALUE ret;
+    GdkPixbuf* dest = gdk_pixbuf_copy(_SELF(self));
+    if (dest == NULL)
+        return Qnil;
+    ret = GOBJ2RVAL(dest);
+    g_object_unref(dest);
+    return ret;
 }
 
 #if RBGDK_PIXBUF_CHECK_VERSION(2,4,0)
@@ -277,49 +289,71 @@ get_file_info(self, filename)
 
 #endif
 
-/****************************************************/
-/* File saving */
 static VALUE
-save(argc, argv, self)
-    int argc;
-    VALUE *argv;
-    VALUE self;
+save_to(VALUE self, gchar *filename, gchar *type, VALUE options)
 {
-    VALUE filename, type, options, key, str;
-    GError* error = NULL;
-    gboolean result;
-    gchar** keys = NULL;
-    gchar** values = NULL;
-    gint len, i;
-    ID to_s = rb_intern("to_s");
-    VALUE ary;
+    VALUE result = self;
+    GError *error = NULL;
+    gchar **keys = NULL;
+    gchar **values = NULL;
 
-    rb_scan_args(argc, argv, "21", &filename, &type, &options);
+    if (!NIL_P(options)) {
+        VALUE ary, key, value;
+        ID to_s;
+        gint len, i;
 
-    if (options != Qnil){
         Check_Type(options, T_HASH);
+        to_s = rb_intern("to_s");
+
         ary = rb_funcall(options, rb_intern("to_a"), 0);
-        len = RARRAY(ary)->len;
-        keys = ALLOCA_N(gchar*, len + 1);
-        values = ALLOCA_N(gchar*, len + 1);
+        len = RARRAY_LEN(ary);
+        keys = ALLOCA_N(gchar *, len + 1);
+        values = ALLOCA_N(gchar *, len + 1);
         for (i = 0; i < len; i++) {
-            key = RARRAY(RARRAY(ary)->ptr[i])->ptr[0];
+            key = RARRAY_PTR(RARRAY_PTR(ary)[i])[0];
             if (SYMBOL_P(key)) {
-               keys[i] = rb_id2name(SYM2ID(key));
+                const char *const_key;
+                const_key = rb_id2name(SYM2ID(key));
+                keys[i] = (gchar *)const_key;
             } else {
                 keys[i] = RVAL2CSTR(key);
             }
-            str = rb_funcall(RARRAY(RARRAY(ary)->ptr[i])->ptr[1], to_s, 0);
-            values[i] = RVAL2CSTR(str);
+            value = rb_funcall(RARRAY_PTR(RARRAY_PTR(ary)[i])[1], to_s, 0);
+            values[i] = RVAL2CSTR(value);
         }
         keys[len] = NULL;
         values[len] = NULL;
     }
-    result = gdk_pixbuf_savev(_SELF(self), RVAL2CSTR(filename),
-                              RVAL2CSTR(type), keys, values, &error);
-    if (! result) RAISE_GERROR(error);
 
-    return self;
+    if (filename) {
+        gdk_pixbuf_savev(_SELF(self), filename, type, keys, values, &error);
+    }
+#if RBGDK_PIXBUF_CHECK_VERSION(2,4,0)
+    else {
+        gchar *buffer;
+        gsize buffer_size;
+        if (gdk_pixbuf_save_to_bufferv(_SELF(self), &buffer, &buffer_size,
+                                       type, keys, values, &error))
+            result = rb_str_new(buffer, buffer_size);
+    }
+#endif
+
+    if (error)
+        RAISE_GERROR(error);
+
+    return result;
+}
+
+/****************************************************/
+/* File saving */
+static VALUE
+save(int argc, VALUE *argv, VALUE self)
+{
+    VALUE filename, type, options;
+
+    rb_scan_args(argc, argv, "21", &filename, &type, &options);
+
+    return save_to(self, RVAL2CSTR(filename), RVAL2CSTR(type), options);
 }
 
 #if RBGDK_PIXBUF_CHECK_VERSION(2,4,0)
@@ -334,49 +368,13 @@ gboolean    gdk_pixbuf_save_to_callbackv    (GdkPixbuf *pixbuf,
 */
 
 static VALUE
-save_to_buffer(argc, argv, self)
-    int argc;
-    VALUE *argv;
-    VALUE self;
+save_to_buffer(int argc, VALUE *argv, VALUE self)
 {
-    VALUE type, options, key, str;
-    GError* error = NULL;
-    gboolean result;
-    gchar** keys = NULL;
-    gchar** values = NULL;
-    gchar* buffer;
-    gsize buffer_size;
-    gint len, i;
-    ID to_s = rb_intern("to_s");
-    VALUE ary;
+    VALUE type, options;
 
     rb_scan_args(argc, argv, "11", &type, &options);
 
-    if (options != Qnil){
-        Check_Type(options, T_HASH);
-        ary = rb_funcall(options, rb_intern("to_a"), 0);
-        len = RARRAY(ary)->len;
-        keys = ALLOCA_N(gchar*, len + 1);
-        values = ALLOCA_N(gchar*, len + 1);
-        for (i = 0; i < len; i++) {
-            key = RARRAY(RARRAY(ary)->ptr[i])->ptr[0];
-            if (SYMBOL_P(key)) {
-               keys[i] = rb_id2name(SYM2ID(key));
-            } else {
-                keys[i] = RVAL2CSTR(key);
-            }
-            str = rb_funcall(RARRAY(RARRAY(ary)->ptr[i])->ptr[1], to_s, 0);
-            values[i] = RVAL2CSTR(str);
-        }
-        keys[len] = NULL;
-        values[len] = NULL;
-    }
-    result = gdk_pixbuf_save_to_bufferv(_SELF(self), &buffer, &buffer_size,
-                                        RVAL2CSTR(type), keys, values, &error);
-
-    if (! result) RAISE_GERROR(error);
-
-    return rb_str_new(buffer, buffer_size);
+    return save_to(self, NULL, RVAL2CSTR(type), options);
 }
 #endif
 
@@ -402,6 +400,9 @@ scale_simple(argc, argv, self)
                                    NUM2INT(dest_width),
                                    NUM2INT(dest_height),
                                    type);
+    if (dest == NULL)
+        return Qnil;
+
     ret = GOBJ2RVAL(dest);
     g_object_unref(dest);
     return ret;
@@ -415,19 +416,19 @@ scale(argc, argv, self)
 {
     GdkInterpType type = GDK_INTERP_BILINEAR;
 
-    VALUE dest, dest_x, dest_y, dest_width, dest_height; 
+    VALUE src, src_x, src_y, src_width, src_height; 
     VALUE offset_x, offset_y, scale_x, scale_y, interp_type;
 
-    rb_scan_args(argc, argv, "91", &dest, &dest_x, &dest_y, 
-                 &dest_width, &dest_height, &offset_x, &offset_y, 
+    rb_scan_args(argc, argv, "91", &src, &src_x, &src_y, 
+                 &src_width, &src_height, &offset_x, &offset_y, 
                  &scale_x, &scale_y, &interp_type);
 
     if (!NIL_P(interp_type))
         type = RVAL2GENUM(interp_type, GDK_TYPE_INTERP_TYPE);
 
-    gdk_pixbuf_scale(_SELF(dest), _SELF(self),
-                     NUM2INT(dest_x), NUM2INT(dest_y), 
-                     NUM2INT(dest_width), NUM2INT(dest_height),
+    gdk_pixbuf_scale(_SELF(src), _SELF(self), 
+                     NUM2INT(src_x), NUM2INT(src_y), 
+                     NUM2INT(src_width), NUM2INT(src_height),
                      NUM2DBL(offset_x), NUM2DBL(offset_y),
                      NUM2DBL(scale_x), NUM2DBL(scale_y), type);
     return self;
@@ -451,6 +452,9 @@ composite_simple(self, dest_width, dest_height, interp_type, overall_alpha,
         type, NUM2INT(overall_alpha), NUM2INT(check_size),
         NUM2UINT(color1), NUM2UINT(color2));
 
+    if (dest == NULL)
+        return Qnil;
+
     ret = GOBJ2RVAL(dest);
     g_object_unref(dest);
     return ret;
@@ -512,14 +516,26 @@ static VALUE
 rotate_simple(self, angle)
     VALUE self, angle;
 {
-    return GOBJ2RVAL(gdk_pixbuf_rotate_simple(_SELF(self), RVAL2GENUM(angle, GDK_TYPE_PIXBUF_ROTATION)));
+    VALUE ret;
+    GdkPixbuf* dest = gdk_pixbuf_rotate_simple(_SELF(self), RVAL2GENUM(angle, GDK_TYPE_PIXBUF_ROTATION));
+    if (dest == NULL)
+        return Qnil;
+    ret = GOBJ2RVAL(dest);
+    g_object_unref(dest);
+    return ret;
 }
 
 static VALUE
 flip(self, horizontal)
     VALUE self, horizontal;
 {
-    return GOBJ2RVAL(gdk_pixbuf_flip(_SELF(self), RTEST(horizontal)));
+    VALUE ret;
+    GdkPixbuf* dest = gdk_pixbuf_flip(_SELF(self), RVAL2CBOOL(horizontal));
+    if (dest == NULL)
+        return Qnil;
+    ret = GOBJ2RVAL(dest);
+    g_object_unref(dest);
+    return ret;
 }
 #endif
 
@@ -527,14 +543,14 @@ static VALUE
 add_alpha(self, substitute_color, r, g, b)
     VALUE self, substitute_color, r, g, b;
 {
-    VALUE ret = Qnil;
-    GdkPixbuf* buf = gdk_pixbuf_add_alpha(_SELF(self),
-                                          RTEST(substitute_color),
-                                          FIX2INT(r), FIX2INT(g), FIX2INT(b));
-    if (buf){
-        ret = GOBJ2RVAL(buf);
-        g_object_unref(buf);
-    }
+    VALUE ret;
+    GdkPixbuf* dest = gdk_pixbuf_add_alpha(_SELF(self),
+                                           RVAL2CBOOL(substitute_color),
+                                           FIX2INT(r), FIX2INT(g), FIX2INT(b));
+    if (dest == NULL)
+        return Qnil;
+    ret = GOBJ2RVAL(dest);
+    g_object_unref(dest);
     return ret;
 }
 
@@ -554,7 +570,7 @@ saturate_and_pixelate(self, staturation, pixelate)
 {
     GdkPixbuf* dest = gdk_pixbuf_copy(_SELF(self));
     gdk_pixbuf_saturate_and_pixelate(_SELF(self), dest, 
-                                     NUM2DBL(staturation), RTEST(pixelate));
+                                     NUM2DBL(staturation), RVAL2CBOOL(pixelate));
     return GOBJ2RVAL(dest);
 }
 
