@@ -6,24 +6,18 @@
 # license of Ruby-GNOME2.
 #
 
-# for dropline GTK2-Runtime DLL
-# http://www.dropline.net/gtk/
+require 'rbconfig'
 if /mingw|mswin|mswin32/ =~ RUBY_PLATFORM
-  begin
-    require 'win32/registry'
-    begin
-      GTK2Dir =
-        Win32::Registry::HKEY_CURRENT_USER.open('Software\GTK\2.0')['Path']
-      ENV['PATH'] = %w(bin lib).collect{|dir|
-        "#{GTK2Dir}\\#{dir};"
-      }.join('') + ENV['PATH']
-    rescue Win32::Registry::Error
-    end
-  rescue LoadError
-  end
+  ENV['PATH'] = %w(bin lib).collect{|dir|
+    "#{Config::CONFIG["prefix"]}\\lib\\GTK\\#{dir};"
+  }.join('') + ENV['PATH']
 end
 
+require 'English'
+require 'thread'
+
 module GLib
+  module_function
   def check_binding_version?(major, minor, micro)
     BINDING_VERSION[0] > major ||
       (BINDING_VERSION[0] == major && 
@@ -32,21 +26,19 @@ module GLib
        BINDING_VERSION[1] == minor &&
        BINDING_VERSION[2] >= micro)  
   end
-  module_function :check_binding_version?
 
-  def exit_application(status)
-    msg = $!.message || $!.to_s
-    msg = $!.class.to_s if msg == ""
-    backtrace = $!.backtrace
+  def exit_application(exception, status)
+    msg = exception.message || exception.to_s
+    msg = exception.class.to_s if msg == ""
+    backtrace = exception.backtrace
     $stderr.puts backtrace.shift + ": #{msg}"
     backtrace.each do |v|
       $stderr.puts "\t from #{v}"
     end
     exit(status)
   end
-  module_function :exit_application
 
-  def self.__add_one_arg_setter(klass)
+  def __add_one_arg_setter(klass)
     #for Instance methods.
     ary = klass.instance_methods(false)
     ary.each do |m|
@@ -163,6 +155,16 @@ module GLib
     end
   end
 
+  if const_defined?(:UserDirectory)
+    class UserDirectory
+      constants.each do |name|
+        if /\ADIRECTORY_/ =~ name
+          const_set($POSTMATCH, const_get(name))
+        end
+      end
+    end
+  end
+
   LOG_DOMAIN = "GLib"
 
   class Object
