@@ -3,13 +3,13 @@
 
   rbglib_iochannel.c -
 
-  $Author: mutoh $
-  $Date: 2005/07/25 16:46:32 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:28 $
 
   Copyright (C) 2005 Masao Mutoh
 ************************************************/
 
-#include "global.h"
+#include "rbgprivate.h"
 
 static ID id_call;
 
@@ -247,7 +247,7 @@ ioc_read_line(argc, argv, self)
         old_line_term = g_io_channel_get_line_term(_SELF(self), &old_line_term_len);
 
         g_io_channel_set_line_term(_SELF(self), RVAL2CSTR(line_term),
-                                   RSTRING(line_term)->len);   
+                                   RSTRING_LEN(line_term));   
     }
 
     status = g_io_channel_read_line(_SELF(self), &str, NULL, NULL, &err);
@@ -285,7 +285,7 @@ ioc_gets(argc, argv, self)
 
         old_line_term = g_io_channel_get_line_term(_SELF(self), &old_line_term_len);
         g_io_channel_set_line_term(_SELF(self), RVAL2CSTR(line_term),
-                                   RSTRING(line_term)->len);   
+                                   RSTRING_LEN(line_term));   
     }
 
     status = g_io_channel_read_line(_SELF(self), &str, NULL, NULL, &err);
@@ -310,14 +310,14 @@ static VALUE
 ioc_set_line_term(args)
     VALUE args;
 {
-    VALUE self = RARRAY(args)->ptr[0];
-    VALUE doit = RARRAY(args)->ptr[1];
-    VALUE line_term = RARRAY(args)->ptr[2];
+    VALUE self = RARRAY_PTR(args)[0];
+    VALUE doit = RARRAY_PTR(args)[1];
+    VALUE line_term = RARRAY_PTR(args)[2];
 
     if (doit == Qtrue){
         StringValue(line_term);
         g_io_channel_set_line_term(_SELF(self), RVAL2CSTR(line_term),
-                                   RSTRING(line_term)->len);  
+                                   RSTRING_LEN(line_term));  
     }
     return self;
 }
@@ -347,7 +347,7 @@ ioc_each_line(argc, argv, self)
 
         old_line_term = g_io_channel_get_line_term(_SELF(self), &old_line_term_len);
         g_io_channel_set_line_term(_SELF(self), RVAL2CSTR(line_term),
-                                   RSTRING(line_term)->len);
+                                   RSTRING_LEN(line_term));
     }
 
     while (TRUE){
@@ -412,7 +412,7 @@ ioc_write_chars(self, buf)
 
     StringValue(buf);
 
-    count = RSTRING(buf)->len;
+    count = RSTRING_LEN(buf);
 
     status = g_io_channel_write_chars(_SELF(self), RVAL2CSTR(buf), count, &bytes_written, &err);
 
@@ -421,7 +421,6 @@ ioc_write_chars(self, buf)
 }
 
 static ID id_unpack;
-static VALUE utf8;
 
 static VALUE
 ioc_write_unichar(self, thechar)
@@ -435,8 +434,8 @@ ioc_write_unichar(self, thechar)
     if (TYPE(thechar) == T_FIXNUM) {
         unichar = NUM2UINT(thechar);
     } else {
-        VALUE ary = rb_funcall(thechar, id_unpack, 1, utf8);
-        unichar = NUM2UINT(RARRAY(ary)->ptr[0]);
+        VALUE ary = rb_funcall(thechar, id_unpack, 1, CSTR2RVAL("U"));
+        unichar = NUM2UINT(RARRAY_PTR(ary)[0]);
     }
 
     status = g_io_channel_write_unichar(_SELF(self), unichar, &err);
@@ -503,7 +502,7 @@ ioc_shutdown(argc, argv, self)
     rb_scan_args(argc, argv, "01", &flush);
 
     if (!NIL_P(flush)){
-        gflush = RTEST(flush);
+        gflush = RVAL2CBOOL(flush);
     }
 
     status = g_io_channel_shutdown(_SELF(self), gflush, &err);
@@ -526,7 +525,7 @@ io_func(source, condition, func)
     GIOCondition condition;
     gpointer func;
 {
-    return RTEST(rb_funcall((VALUE)func, id_call, 2, 
+    return RVAL2CBOOL(rb_funcall((VALUE)func, id_call, 2, 
                             BOXED2RVAL(source, G_TYPE_IO_CHANNEL), 
                             INT2NUM(condition)));
 }
@@ -535,7 +534,7 @@ static VALUE
 ioc_add_watch(self, condition)
     VALUE self, condition;
 {
-    VALUE func = G_BLOCK_PROC();
+    VALUE func = rb_block_proc();
     G_RELATIVE(self, func);
     return UINT2NUM(g_io_add_watch(_SELF(self), NUM2INT(condition), 
                                    (GIOFunc)io_func, (gpointer)func));
@@ -613,7 +612,7 @@ ioc_set_line_term(self, line_term)
 {
     StringValue(line_term);
     g_io_channel_set_line_term(_SELF(self), RVAL2CSTR(line_term),
-                               RSTRING(line_term)->len);
+                               RSTRING_LEN(line_term));
     return self;
 }
 */
@@ -629,7 +628,7 @@ static VALUE
 ioc_set_buffered(self, buffered)
     VALUE self, buffered;
 {
-    g_io_channel_set_buffered(_SELF(self), RTEST(buffered));
+    g_io_channel_set_buffered(_SELF(self), RVAL2CBOOL(buffered));
     return self;
 }
 
@@ -702,8 +701,8 @@ ioc_puts_ary(ary, out, recur)
     VALUE tmp;
     long i;
 
-    for (i=0; i<RARRAY(ary)->len; i++) {
-        tmp = RARRAY(ary)->ptr[i];
+    for (i=0; i<RARRAY_LEN(ary); i++) {
+        tmp = RARRAY_PTR(ary)[i];
         if (recur) {
             tmp = rb_str_new2("[...]");
         }
@@ -747,8 +746,8 @@ ioc_puts(argc, argv, self)
           line = rb_obj_as_string(argv[i]);
         }
         ioc_write_chars(self, line);
-        if (RSTRING(line)->len == 0 ||
-            RSTRING(line)->ptr[RSTRING(line)->len-1] != '\n') {
+        if (RSTRING_LEN(line) == 0 ||
+            RSTRING_PTR(line)[RSTRING_LEN(line)-1] != '\n') {
             ioc_write_chars(self, rb_default_rs);
         }
     }
@@ -802,7 +801,6 @@ Init_glib_io_channel()
 
     id_call = rb_intern("call");
     id_unpack = rb_intern("unpack");
-    utf8 = CSTR2RVAL("U");
 
 
     rb_define_method(io, "initialize", ioc_initialize, -1);
