@@ -3,51 +3,28 @@
 
   rbgobj_closure.c -
 
-  $Author: mutoh $
-  $Date: 2006/06/17 09:18:12 $
+  $Author: ggc $
+  $Date: 2007/07/13 14:27:06 $
 
   Copyright (C) 2002-2006  Ruby-GNOME2 Project
   Copyright (C) 2002,2003  Masahiro Sakai
 
 **********************************************************************/
 
-#include "global.h"
-#ifdef G_OS_WIN32
-#undef HAVE_NATIVETHREAD /* FIXME */
-#else
-#include <unistd.h>
-#endif
+#include "rbgprivate.h"
 
-#ifdef HAVE_YARV_H
-#include "yarv.h"
-#define ruby_errinfo (GET_THREAD()->errinfo)
-#undef HAVE_NATIVETHREAD /* FIXME */
-#endif
-
-static ID id_call;
-static ID id_holder;
+static ID id_call, id_closures;
 static gboolean rclosure_initialized = FALSE;
 
-#define CALLBACK_KEY       rb_str_new2("callback")
-#define EXTRA_ARGS_KEY     rb_str_new2("extra_args")
-
-#define HOLDER(closure)       \
-  (rb_ivar_get((closure)->rb_holder, id_holder))
-#define SETUP_HOLDER(closure) \
-  (rb_ivar_set((closure)->rb_holder, id_holder, rb_hash_new()))
-
 typedef struct _GRClosure GRClosure;
-
-typedef struct _GRClosureHolder
-{
-    GRClosure *closure;
-} GRClosureHolder;
-
 struct _GRClosure
 {
     GClosure closure;
+    VALUE callback;
+    VALUE extra_args;
     VALUE rb_holder;
     gint count;
+    GList *objects;
     GValToRValSignalFunc g2r_func;
 };
 
@@ -76,7 +53,7 @@ struct marshal_arg
 static int
 rclosure_alive_p(GRClosure *rclosure)
 {
-    return rclosure->count > 0;
+    return (rclosure->count > 0 && !NIL_P(rclosure->rb_holder));
 }
 
 static VALUE
@@ -111,8 +88,8 @@ rclosure_marshal_do(VALUE arg_)
 
     if (rclosure_alive_p(rclosure)) {
         VALUE callback, extra_args;
-        callback = rb_hash_aref(HOLDER(rclosure), CALLBACK_KEY);
-        extra_args = rb_hash_aref(HOLDER(rclosure), EXTRA_ARGS_KEY);
+        callback = rclosure->callback;
+        extra_args = rclosure->extra_args;
 
         if (!NIL_P(extra_args)) {
             args = rb_ary_concat(args, extra_args);
@@ -129,70 +106,6 @@ rclosure_marshal_do(VALUE arg_)
     return Qnil;
 }
 
-#ifdef HAVE_NATIVETHREAD
-static GStaticMutex callback_init_mutex = G_STATIC_MUTEX_INIT;
-static GMutex *callback_mutex = NULL;
-static GCond *callback_done_cond = NULL;
-static GCond *callback_finished_cond = NULL;
-static struct marshal_arg* m_arg = NULL;
-static int callback_fd[2];
-
-static VALUE
-rclosure_marshal_pop(void) {
- for (;;) {
-   char buf[1];
-
-   /* wait untill we're triggered. If this happens we can read from the pipe
-    * and it's guaranteed that the needed mutexes are initialized */
-   rb_thread_wait_fd(callback_fd[0]);
-   read(callback_fd[0], buf, 1);
-
-   g_mutex_lock(callback_mutex);
-   if (m_arg) {
-       G_PROTECT_CALLBACK(rclosure_marshal_do, m_arg);
-       m_arg = NULL;
-   }
-   g_cond_signal(callback_done_cond);
-   g_mutex_unlock(callback_mutex);
- }
-}
-
-static void
-init_callback_mutex(void) {
-  if (callback_mutex == NULL) {
-    g_static_mutex_lock(&callback_init_mutex);
-    if (callback_mutex == NULL) {
-      callback_mutex = g_mutex_new();
-      callback_done_cond = g_cond_new();
-      callback_finished_cond = g_cond_new();
-    } /* else some other thread initialised it */
-
-    g_static_mutex_unlock(&callback_init_mutex);
-  }
-}
-
-static void
-rclosure_marshal_push(struct marshal_arg* arg) {
-  init_callback_mutex();
-
-  g_mutex_lock(callback_mutex);
-  while (m_arg) {
-    /* Wait for another callback to finish up */
-    g_cond_wait(callback_finished_cond, callback_mutex);
-  }
-  m_arg = arg;
-  /* trigger ruby callback thread */
-  write(callback_fd[1],"c", 1);
-
-  /* Wait until the ruby callback thread signals is done and then signal
-   * waiting callback pushes that we're finished 
-   */
-  g_cond_wait(callback_done_cond, callback_mutex);
-  g_cond_signal(callback_finished_cond);
-  g_mutex_unlock(callback_mutex);
-}
-#endif
-
 static void
 rclosure_marshal(GClosure*       closure,
                  GValue*         return_value,
@@ -215,43 +128,68 @@ rclosure_marshal(GClosure*       closure,
     arg.invocation_hint = invocation_hint;
     arg.marshal_data    = marshal_data;
 
-#ifdef HAVE_NATIVETHREAD
-    if (!is_ruby_native_thread()) {
-        if (!g_thread_supported()) {
-            rb_bug("glib signal in another thread, but gthreads not supported");
-        }
-        rclosure_marshal_push(&arg);
-    } else {
-       G_PROTECT_CALLBACK(rclosure_marshal_do, &arg);
-    }
-#else
     G_PROTECT_CALLBACK(rclosure_marshal_do, &arg);
-#endif
 }
 
+static void rclosure_weak_notify(gpointer data, GObject* where_the_object_was);
+
+static void
+rclosure_unref(GRClosure *rclosure)
+{
+    rclosure->count--;
+
+    if (!rclosure_alive_p(rclosure)) {
+        GList *next;
+        for (next = rclosure->objects; next; next = next->next) {
+            GObject *object = G_OBJECT(next->data);
+            g_object_weak_unref(object, rclosure_weak_notify, rclosure);
+        }
+        g_list_free(rclosure->objects);
+        rclosure->objects = NULL;
+        if (!NIL_P(rclosure->rb_holder)) {
+            RDATA(rclosure->rb_holder)->dmark = NULL;
+            RDATA(rclosure->rb_holder)->dfree = NULL;
+            rclosure->rb_holder = Qnil;
+        }
+    }
+}
 
 static void
 rclosure_invalidate(gpointer data, GClosure* closure)
 {
     GRClosure *rclosure = (GRClosure*)closure;
 
-    if (rclosure_alive_p(rclosure)) {
-        if (rclosure->count != 1) {
-            rb_warn("GRClosure invalidate: strange reference count: %d\n",
-                    rclosure->count);
+    if (rclosure->count > 0) {
+        GList *next;
+
+        rclosure->count = 1;
+        for (next = rclosure->objects; next; next = next->next) {
+            GObject *object = G_OBJECT(next->data);
+            VALUE obj = rbgobj_ruby_object_from_instance2(object, FALSE);
+            if (!NIL_P(rclosure->rb_holder) && !NIL_P(obj))
+                G_REMOVE_RELATIVE(obj, id_closures, rclosure->rb_holder);
         }
-        rclosure->count = 0;
+
+        rclosure_unref(rclosure);
     }
 }
 
 static void
-gr_closure_holder_free(GRClosureHolder *holder)
+gr_closure_holder_mark(GRClosure *rclosure)
 {
-    if (holder) {
-        if (rclosure_alive_p(holder->closure)) {
-            holder->closure->count--;
-        }
-        free(holder);
+    rb_gc_mark(rclosure->callback);
+    rb_gc_mark(rclosure->extra_args);
+}
+
+static void
+gr_closure_holder_free(GRClosure *rclosure)
+{
+    if (rclosure->count > 0) {
+        rclosure->count = 1;
+
+        /* No need to remove us from the relatives hash of our objects, as
+         * those aren't alive anymore anyway */
+        rclosure_unref(rclosure);
     }
 }
 
@@ -259,20 +197,18 @@ GClosure*
 g_rclosure_new(VALUE callback_proc, VALUE extra_args, GValToRValSignalFunc g2r_func)
 {
     GRClosure* closure;
-    GRClosureHolder *holder;
 
     closure = (GRClosure*)g_closure_new_simple(sizeof(GRClosure), NULL);
 
-    closure->count      = 0;
+    closure->count      = 1;
     closure->g2r_func   = g2r_func;
-    closure->rb_holder  = Data_Make_Struct(rb_cData, GRClosureHolder, 0,
-                                           gr_closure_holder_free, holder);
-
-    holder->closure = closure;
-    SETUP_HOLDER(closure);
-
-    rb_hash_aset(HOLDER(closure), CALLBACK_KEY, callback_proc);
-    rb_hash_aset(HOLDER(closure), EXTRA_ARGS_KEY, extra_args);
+    closure->objects    = NULL;
+    closure->callback   = callback_proc;
+    closure->extra_args = extra_args;
+    closure->rb_holder  = Data_Wrap_Struct(rb_cData,
+                                           gr_closure_holder_mark,
+                                           gr_closure_holder_free,
+                                           closure);
 
     g_closure_set_marshal((GClosure*)closure, &rclosure_marshal);
     g_closure_add_invalidate_notifier((GClosure*)closure, NULL,
@@ -286,7 +222,9 @@ rclosure_weak_notify(gpointer data, GObject* where_the_object_was)
 {
     GRClosure *rclosure = data;
     if (rclosure_alive_p(rclosure)) {
-        rclosure->count--;
+        rclosure->objects =
+            g_list_remove(rclosure->objects, where_the_object_was);
+        rclosure_unref(rclosure);
     }
 }
 
@@ -296,15 +234,17 @@ g_rclosure_attach(GClosure *closure, VALUE object)
     static VALUE mGLibObject = (VALUE)NULL;
     GRClosure *rclosure = (GRClosure *)closure;
 
-    G_CHILD_ADD(object, rclosure->rb_holder);
-
-    rclosure->count++;
+    G_RELATIVE2(object, Qnil, id_closures, rclosure->rb_holder);
 
     if (!mGLibObject) {
         mGLibObject = rb_const_get(mGLib, rb_intern("Object"));
     }
     if (rb_obj_is_kind_of(object, mGLibObject)) {
-        g_object_weak_ref(RVAL2GOBJ(object), rclosure_weak_notify, rclosure);
+        GObject *gobject;
+        gobject = RVAL2GOBJ(object);
+        rclosure->count++;
+        g_object_weak_ref(gobject, rclosure_weak_notify, rclosure);
+        rclosure->objects = g_list_prepend(rclosure->objects, gobject);
     }
 }
 
@@ -318,19 +258,7 @@ static void
 Init_rclosure()
 {
     id_call = rb_intern("call");
-    id_holder = rb_intern("holder");
-
-#ifdef HAVE_NATIVETHREAD
-    /* startup the ruby thread to pull callbacks from other threads */
-    {
-        static VALUE thread;
-        if (pipe(callback_fd) != 0)
-            rb_bug("Unable to create glib callback thread\n");
-        thread = rb_thread_create(rclosure_marshal_pop, NULL);
-        rb_global_variable(&thread);
-    }
-#endif
-
+    id_closures = rb_intern("closures");
     rclosure_initialized = TRUE;
     rb_set_end_proc(rclosure_end_proc, Qnil);
 }
@@ -341,7 +269,7 @@ static VALUE
 closure_initialize(self)
     VALUE self;
 {
-    GClosure* closure = g_rclosure_new(G_BLOCK_PROC(), Qnil, NULL);
+    GClosure* closure = g_rclosure_new(rb_block_proc(), Qnil, NULL);
     G_INITIALIZE(self, closure);
     g_closure_sink(closure);
     return self;
@@ -352,7 +280,7 @@ closure_in_marshal(self)
     VALUE self;
 {
     GClosure* closure = RVAL2BOXED(self, G_TYPE_CLOSURE);
-    return closure->in_marshal ? Qtrue : Qfalse;
+    return CBOOL2RVAL(closure->in_marshal);
 }
 
 static VALUE
@@ -360,7 +288,7 @@ closure_is_invalid(self)
     VALUE self;
 {
     GClosure* closure = RVAL2BOXED(self, G_TYPE_CLOSURE);
-    return closure->is_invalid ? Qtrue : Qfalse;
+    return CBOOL2RVAL(closure->is_invalid);
 }
 
 static VALUE
