@@ -3,8 +3,8 @@
 
   rbgobj_object.c -
 
-  $Author: mutoh $
-  $Date: 2006/08/17 12:46:22 $
+  $Author: sakai $
+  $Date: 2007/07/19 22:03:44 $
 
   Copyright (C) 2002-2004  Ruby-GNOME2 Project Team
   Copyright (C) 2002-2003  Masahiro Sakai
@@ -17,31 +17,167 @@
 
 **********************************************************************/
 
-#include "ruby.h"
-#include "st.h"
-#include "global.h"
+#include "rbgprivate.h"
 
+VALUE rbgobj_cObject;
 static VALUE eNoPropertyError;
+static GQuark RUBY_GOBJECT_OBJ_KEY;
 
+/* deperecated */
 void
 rbgobj_add_abstract_but_create_instance_class(gtype)
     GType gtype;
 {
-    RGObjClassInfo* cinfo = (RGObjClassInfo*)GTYPE2CINFO(gtype);
-    cinfo->flags |= RBGOBJ_ABSTRACT_BUT_CREATABLE;
+}
+
+static void
+weak_notify(gpointer data, GObject *where_the_object_was)
+{
+    gobj_holder *holder = data;
+
+    rbgobj_instance_call_cinfo_free(holder->gobj);
+    rbgobj_invalidate_relatives(holder->self);
+    holder->destroyed = TRUE;
+
+    g_object_unref(holder->gobj);
+    holder->gobj = NULL;
+}
+
+static void
+holder_mark(gobj_holder *holder)
+{
+    if (holder->gobj && !holder->destroyed)
+        rbgobj_instance_call_cinfo_mark(holder->gobj);
+}
+
+static void
+holder_free(gobj_holder *holder)
+{
+    if (holder->gobj) {
+        if (!holder->destroyed) {
+            g_object_set_qdata(holder->gobj, RUBY_GOBJECT_OBJ_KEY, NULL);
+            g_object_weak_unref(holder->gobj, (GWeakNotify)weak_notify, holder);
+	    weak_notify(holder, holder->gobj);
+        }
+        holder->gobj = NULL;
+    }
+    xfree(holder);
 }
 
 static VALUE
 gobj_s_allocate(klass)
     VALUE klass;
 {
-    const RGObjClassInfo* cinfo = rbgobj_lookup_class(klass);
-    if (G_TYPE_IS_ABSTRACT(cinfo->gtype) &&
-        !(cinfo->flags & RBGOBJ_ABSTRACT_BUT_CREATABLE))
-        rb_raise(rb_eTypeError, "abstract class");
-    return rbgobj_create_object(klass);
+    gobj_holder* holder;
+    VALUE result;
+
+    result = Data_Make_Struct(klass, gobj_holder, holder_mark, holder_free, holder);
+    holder->self  = result;
+    holder->gobj  = NULL;
+    holder->cinfo = NULL;
+    holder->destroyed = FALSE;
+
+    return result;
 }
 
+/* deprecated */
+VALUE
+rbgobj_create_object(klass)
+    VALUE klass;
+{
+    return gobj_s_allocate(klass);
+}
+
+void
+rbgobj_gobject_initialize(obj, cobj)
+    VALUE obj;
+    gpointer cobj;
+{
+    gobj_holder* holder = g_object_get_qdata((GObject*)cobj, RUBY_GOBJECT_OBJ_KEY);
+    if (holder)
+        rb_raise(rb_eRuntimeError, "ruby wrapper for this GObject* already exists.");
+    Data_Get_Struct(obj, gobj_holder, holder);
+    holder->cinfo = RVAL2CINFO(obj);
+    holder->gobj  = (GObject*)cobj;
+    holder->destroyed = FALSE;
+
+    g_object_set_qdata((GObject*)cobj, RUBY_GOBJECT_OBJ_KEY, (gpointer)holder);
+    g_object_weak_ref((GObject*)cobj, (GWeakNotify)weak_notify, holder);
+    {
+        GType t1 = G_TYPE_FROM_INSTANCE(cobj);
+        GType t2 = CLASS2GTYPE(CLASS_OF(obj));
+
+        if (t1 != t2) {
+            if (!g_type_is_a(t1, t2))
+                rb_raise(rb_eTypeError, "%s is not subtype of %s",
+                         g_type_name(t1), g_type_name(t2));
+        }
+    }
+}
+
+VALUE
+rbgobj_get_ruby_object_from_gobject(GObject* gobj, gboolean alloc)
+{
+    gobj_holder *holder;
+
+    holder = g_object_get_qdata(gobj, RUBY_GOBJECT_OBJ_KEY);
+    if (holder) {
+        return holder->self;
+    } else if (alloc) {
+        VALUE obj;
+
+	obj = gobj_s_allocate(GTYPE2CLASS(G_OBJECT_TYPE(gobj)));
+        gobj = g_object_ref(gobj);
+        rbgobj_gobject_initialize(obj, (gpointer)gobj);
+        return obj;
+    } else {
+        return Qnil;
+    }
+}
+
+GObject*
+rbgobj_get_gobject(obj)
+    VALUE obj;
+{
+    gobj_holder* holder;
+
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(obj, GTYPE2CLASS(G_TYPE_OBJECT))))
+        rb_raise(rb_eTypeError, "not a GLib::Object");
+
+    Data_Get_Struct(obj, gobj_holder, holder);
+
+    if (holder->destroyed)
+        rb_raise(rb_eTypeError, "destroyed GLib::Object");
+    if (!holder->gobj)
+        rb_raise(rb_eTypeError, "uninitialize GLib::Object");
+
+    return holder->gobj;
+}
+
+static VALUE
+dummy_init(argc, argv, self)
+    int argc;
+    VALUE *argv;
+    VALUE self;
+{
+    GType gtype = CLASS2GTYPE(CLASS_OF(self));
+    if (G_TYPE_IS_ABSTRACT(gtype))
+        rb_raise(rb_eTypeError, "initializing abstract class");
+    else
+        return rb_call_super(argc, argv);
+}
+
+void
+rbgobj_init_object_class(klass)
+    VALUE klass;
+{
+    rbgobj_define_property_accessors(klass);
+    if (G_TYPE_IS_ABSTRACT(CLASS2GTYPE(klass)))
+        rb_define_method(klass, "initialize", dummy_init, -1);
+}
+
+/**********************************************************************/
+
 static gboolean
 is_gtkobject(gobj)
     GObject* gobj;
@@ -116,6 +252,113 @@ gobj_s_gobject_new(argc, argv, self)
     return result;
 }
 
+struct param_setup_arg {
+    GObjectClass* gclass;
+    GParameter* params;
+    guint param_size;
+    VALUE params_hash;
+};
+
+static VALUE
+_each_with_index(obj)
+    VALUE obj;
+{
+    rb_funcall(obj, rb_intern("each_with_index"), 0);
+    return Qnil;
+}
+
+static VALUE
+_params_setup(arg, param_setup_arg)
+    VALUE arg;
+    struct param_setup_arg* param_setup_arg;
+{
+    int n = NUM2INT(rb_ary_entry(arg, 1));
+    VALUE name, val;
+    GParamSpec* pspec;
+
+    if (n >= param_setup_arg->param_size)
+       rb_raise(rb_eArgError, "too many parameters");
+
+    arg = rb_ary_entry(arg, 0);
+
+    name = rb_ary_entry(arg, 0);
+    val  = rb_ary_entry(arg, 1);
+
+    if (SYMBOL_P(name))
+        param_setup_arg->params[n].name = rb_id2name(SYM2ID(name));
+    else
+        param_setup_arg->params[n].name = StringValuePtr(name);
+
+    pspec = g_object_class_find_property(
+        param_setup_arg->gclass,
+        param_setup_arg->params[n].name);
+    if (!pspec)
+        rb_raise(rb_eArgError, "No such property: %s",
+                 param_setup_arg->params[n].name);
+
+    g_value_init(&(param_setup_arg->params[n].value),
+                 G_PARAM_SPEC_VALUE_TYPE(pspec));
+    rbgobj_rvalue_to_gvalue(val, &(param_setup_arg->params[n].value));
+    
+    return Qnil;
+}
+
+static VALUE
+gobj_new_body(struct param_setup_arg* arg)
+{
+    rb_iterate((VALUE(*)_((VALUE)))_each_with_index, (VALUE)arg->params_hash, _params_setup, (VALUE)arg);
+    return (VALUE)g_object_newv(G_TYPE_FROM_CLASS(arg->gclass),
+                                arg->param_size, arg->params);
+}
+
+static VALUE
+gobj_new_ensure(struct param_setup_arg* arg)
+{
+    int i;
+    g_type_class_unref(arg->gclass);
+    for (i = 0; i < arg->param_size; i++) {
+        if (G_IS_VALUE(&arg->params[i].value))
+            g_value_unset(&arg->params[i].value);
+    }
+    return Qnil;
+}
+
+GObject*
+rbgobj_gobject_new(gtype, params_hash)
+    GType gtype;
+    VALUE params_hash;
+{
+    GObject* result;
+
+    if (!g_type_is_a(gtype, G_TYPE_OBJECT))
+        rb_raise(rb_eArgError,
+                 "type \"%s\" is not descendant of GObject",
+                 g_type_name(gtype));
+
+    if (NIL_P(params_hash)) {
+        result = g_object_newv(gtype, 0, NULL);
+    } else {
+        size_t param_size;
+        struct param_setup_arg arg;
+
+        param_size = NUM2INT(rb_funcall(params_hash, rb_intern("length"), 0)); 
+
+        arg.param_size = param_size;
+        arg.gclass = G_OBJECT_CLASS(g_type_class_ref(gtype));
+        arg.params = ALLOCA_N(GParameter, param_size);
+        memset(arg.params, 0, sizeof(GParameter) * param_size);
+        arg.params_hash = params_hash;
+
+        result = (GObject*)rb_ensure(&gobj_new_body, (VALUE)&arg,
+                                     &gobj_new_ensure, (VALUE)&arg);
+    }
+
+    if (!result)
+        rb_raise(rb_eRuntimeError, "g_object_newv failed");
+
+    return result;
+}
+
 static VALUE
 gobj_s_install_property(int argc, VALUE* argv, VALUE self)
 {
@@ -151,19 +394,17 @@ gobj_s_property(self, property_name)
     GParamSpec* prop;
     VALUE result;
 
-    if (SYMBOL_P(property_name)) {
+    if (SYMBOL_P(property_name))
         name = rb_id2name(SYM2ID(property_name));
-    } else {
-        StringValue(property_name);
+    else
         name = StringValuePtr(property_name);
-    }
 
     oclass = g_type_class_ref(CLASS2GTYPE(self));
 
     prop = g_object_class_find_property(oclass, name);
     if (!prop){
         g_type_class_unref(oclass);
-        rb_raise(eNoPropertyError, "no such property: %s", name);
+        rb_raise(eNoPropertyError, "No such property: %s", name);
     }
 
     result = GOBJ2RVAL(prop);
@@ -188,7 +429,7 @@ gobj_s_properties(int argc, VALUE* argv, VALUE self)
 
     ary = rb_ary_new();
     for (i = 0; i < n_properties; i++){
-        if (RTEST(inherited_too)
+        if (RVAL2CBOOL(inherited_too)
             || GTYPE2CLASS(props[i]->owner_type) == self)
             rb_ary_push(ary, rb_str_new2(props[i]->name));
     }
@@ -218,7 +459,7 @@ rbgobj_register_property_setter(gtype, name, func)
     oclass = g_type_class_ref(gtype);
     pspec = g_object_class_find_property(oclass, name);
     
-    rb_hash_aset(table, rb_intern(g_param_spec_get_name(pspec)),
+    rb_hash_aset(table, rb_str_new2(g_param_spec_get_name(pspec)),
                  Data_Wrap_Struct(rb_cData, NULL, NULL, func));
 
     g_type_class_unref(oclass);
@@ -242,7 +483,7 @@ rbgobj_register_property_getter(gtype, name, func)
     oclass = g_type_class_ref(gtype);
     pspec = g_object_class_find_property(oclass, name);
 
-    rb_hash_aset(table, rb_intern(g_param_spec_get_name(pspec)),
+    rb_hash_aset(table, rb_str_new2(g_param_spec_get_name(pspec)),
                  Data_Wrap_Struct(rb_cData, NULL, NULL, func));
 }
 
@@ -253,12 +494,10 @@ gobj_set_property(self, prop_name, val)
     GParamSpec* pspec;
     const char* name;
 
-    if (SYMBOL_P(prop_name)) {
+    if (SYMBOL_P(prop_name))
         name = rb_id2name(SYM2ID(prop_name));
-    } else {
-        StringValue(prop_name);
+    else
         name = StringValuePtr(prop_name);
-    }
 
     pspec = g_object_class_find_property(G_OBJECT_GET_CLASS(RVAL2GOBJ(self)),
                                          name);
@@ -303,12 +542,10 @@ gobj_get_property(self, prop_name)
     GParamSpec* pspec;
     const char* name;
 
-    if (SYMBOL_P(prop_name)) {
+    if (SYMBOL_P(prop_name))
         name = rb_id2name(SYM2ID(prop_name));
-    } else {
-        StringValue(prop_name);
+    else
         name = StringValuePtr(prop_name);
-    }
 
     pspec = g_object_class_find_property(G_OBJECT_GET_CLASS(RVAL2GOBJ(self)),
                                          name);
@@ -325,7 +562,7 @@ gobj_get_property(self, prop_name)
             VALUE table = rb_hash_aref(type_to_prop_getter_table,
                                        INT2FIX(pspec->owner_type));
             if (!NIL_P(table)){
-                VALUE obj = rb_hash_aref(table, rb_intern(g_param_spec_get_name(pspec)));
+                VALUE obj = rb_hash_aref(table, CSTR2RVAL(g_param_spec_get_name(pspec)));
                 if (!NIL_P(obj))
                     Data_Get_Struct(obj, void, getter);
             }
@@ -360,7 +597,6 @@ static VALUE
 gobj_notify(self, property_name)
     VALUE self, property_name;
 {
-    StringValue(property_name);
     g_object_notify(RVAL2GOBJ(self), StringValuePtr(property_name));
     return self;
 }
@@ -379,12 +615,12 @@ gobj_is_destroyed(self)
 {
     gobj_holder* holder;
 
-    if (!RTEST(rb_obj_is_kind_of(self, GTYPE2CLASS(G_TYPE_OBJECT))))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(self, GTYPE2CLASS(G_TYPE_OBJECT))))
         rb_raise(rb_eTypeError, "not a GLib::Object");
 
     Data_Get_Struct(self, gobj_holder, holder);
 
-    return (holder->destroyed) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(holder->destroyed);
 }
 
 static VALUE
@@ -392,16 +628,18 @@ gobj_inspect(self)
     VALUE self;
 {
     gobj_holder* holder;
-    char *cname = rb_class2name(CLASS_OF(self));
+    const char *class_name;
     char *s;
     VALUE result;
 
     Data_Get_Struct(self, gobj_holder, holder);
 
+    class_name = rb_class2name(CLASS_OF(self));
     if (!holder->destroyed)
-        s = g_strdup_printf("#<%s:%p ptr=%p>", cname, (void *)self, holder->gobj);
+        s = g_strdup_printf("#<%s:%p ptr=%p>", class_name, (void *)self,
+			    holder->gobj);
     else
-        s = g_strdup_printf("#<%s:%p destroyed>", cname, (void *)self);
+        s = g_strdup_printf("#<%s:%p destroyed>", class_name, (void *)self);
 
     result = rb_str_new2(s);
     g_free(s);
@@ -436,18 +674,13 @@ gobj_initialize(argc, argv, self)
     return Qnil;
 }
 
-static VALUE
-null()
-{
-    return (VALUE)NULL;
-}
-
 static VALUE
 gobj_ref_count(self)
     VALUE self;
 {
-    GObject* gobj = (GObject*)rb_rescue((VALUE(*)())rbgobj_get_gobject, self, null, 0);
-    return INT2NUM(gobj ? gobj->ref_count : 0);
+    gobj_holder* holder;
+    Data_Get_Struct(self, gobj_holder, holder);
+    return INT2NUM(holder->gobj ? holder->gobj->ref_count : 0);
 }
 
 /**********************************************************************/
@@ -545,11 +778,11 @@ type_register(int argc, VALUE* argv, VALUE self)
         parent_type = cinfo->gtype;
     }
 
-    if (NIL_P(type_name)){
+    if (NIL_P(type_name)) {
         VALUE s = rb_funcall(self, rb_intern("name"), 0);
 
         if (strlen(StringValuePtr(s)) == 0)
-            rb_raise(rb_eTypeError, "can't determine type name");        
+            rb_raise(rb_eTypeError, "can't determine type name");
 
         type_name = rb_funcall(
             rb_eval_string("lambda{|x| x.gsub(/::/,'') }"),
@@ -618,16 +851,19 @@ Init_gobject_subclass()
 void 
 Init_gobject_gobject()
 {
-    VALUE cGObject = G_DEF_CLASS2(G_TYPE_OBJECT, "Object", mGLib, gobj_mark, NULL);
+    VALUE cGObject;
+
+    rbgobj_cObject = G_DEF_CLASS_WITH_GC_FUNC(G_TYPE_OBJECT, "Object", mGLib,
+					      gobj_mark, NULL);
+    cGObject = rbgobj_cObject;
+
 #ifdef G_TYPE_INITIALLY_UNOWNED
     G_DEF_CLASS(G_TYPE_INITIALLY_UNOWNED, "InitiallyUnowned", mGLib);
 #endif
 
-#ifndef HAVE_RB_DEFINE_ALLOC_FUNC
-    rb_define_singleton_method(cGObject, "allocate", &gobj_s_allocate, 0);
-#else
+    RUBY_GOBJECT_OBJ_KEY = g_quark_from_static_string("__ruby_gobject_object__");
+
     rb_define_alloc_func(cGObject, (VALUE(*)_((VALUE)))gobj_s_allocate);
-#endif
     rb_define_singleton_method(cGObject, "new!", gobj_s_gobject_new, -1);
 
     rb_define_singleton_method(cGObject, "property", &gobj_s_property, 1);
@@ -639,6 +875,7 @@ Init_gobject_gobject()
     rb_define_method(cGObject, "set_property", gobj_set_property, 2);
     rb_define_method(cGObject, "get_property", gobj_get_property, 1);
     rb_define_method(cGObject, "freeze_notify", gobj_freeze_notify, 0);
+    rb_undef_method(cGObject, "notify");
     rb_define_method(cGObject, "notify", gobj_notify, 1);
     rb_define_method(cGObject, "thaw_notify", gobj_thaw_notify, 0);
     rb_define_method(cGObject, "destroyed?", gobj_is_destroyed, 0);
