@@ -1,3 +1,4 @@
+/* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
 /*****************************************************************************
  *
  * gnomevfs-file.c: GnomeVFS::File class.
@@ -18,9 +19,9 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
  *
- * $Author: sakai $
+ * $Author: ggc $
  *
- * $Date: 2006/06/13 16:03:48 $
+ * $Date: 2007/07/13 16:07:30 $
  *
  *****************************************************************************/
 
@@ -76,7 +77,7 @@ static VALUE
 file_unlink(self, uri)
      VALUE self, uri;
 {
-  if (RTEST(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
+  if (RVAL2CBOOL(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
     return GVFSRESULT2RVAL(
                            gnome_vfs_unlink_from_uri(RVAL2GVFSURI(uri)));
   } else {
@@ -94,13 +95,13 @@ file_move(argc, argv, self)
   gboolean force;
 
   if (rb_scan_args(argc, argv, "21", &source, &target, &r_force) == 3) {
-    force = RTEST(r_force);
+    force = RVAL2CBOOL(r_force);
   } else {
     force = TRUE;
   }
 
   /* XXX: this needs to be fixed */
-  if (RTEST(rb_obj_is_kind_of(source, g_gvfs_uri))) {
+  if (RVAL2CBOOL(rb_obj_is_kind_of(source, g_gvfs_uri))) {
     return GVFSRESULT2RVAL(
                            gnome_vfs_move_uri(RVAL2GVFSURI(source),
                                               RVAL2GVFSURI(target),
@@ -119,14 +120,14 @@ create_symbolic_link(self, uri, reference)
   GnomeVFSURI *tmp;
   VALUE result;
 
-  if (RTEST(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
+  if (RVAL2CBOOL(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
     tmp = RVAL2GVFSURI(uri);
     gnome_vfs_uri_ref(tmp);
   } else {
     tmp = gnome_vfs_uri_new(RVAL2CSTR(uri));
   }
 
-  if (RTEST(rb_obj_is_kind_of(reference, g_gvfs_uri))) {
+  if (RVAL2CBOOL(rb_obj_is_kind_of(reference, g_gvfs_uri))) {
     gchar *str;
 
     str = gnome_vfs_uri_to_string(RVAL2GVFSURI(reference),
@@ -155,9 +156,9 @@ apply_set_info(paths, info, mask)
   GnomeVFSResult result;
 
   Check_Type(paths, T_ARRAY);
-  for (i = 0, n = RARRAY(paths)->len; i < n; i++) {
+  for (i = 0, n = RARRAY_LEN(paths); i < n; i++) {
     path = rb_ary_entry(paths, i);
-    if (RTEST(rb_obj_is_kind_of(path, g_gvfs_uri))) {
+    if (RVAL2CBOOL(rb_obj_is_kind_of(path, g_gvfs_uri))) {
       result = GVFSRESULT2RVAL(
                                gnome_vfs_set_file_info_uri(RVAL2GVFSURI(path),
                                                            info, mask));
@@ -243,7 +244,7 @@ file_m_lstat(argc, argv, self)
 
   info = gnome_vfs_file_info_new();
 
-  if (RTEST(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
+  if (RVAL2CBOOL(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
     result = gnome_vfs_get_file_info_uri(RVAL2GVFSURI(uri),
                                          info,
                                          options);
@@ -276,7 +277,7 @@ file_m_stat(argc, argv, self)
 
   info = gnome_vfs_file_info_new();
 
-  if (RTEST(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
+  if (RVAL2CBOOL(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
     result = gnome_vfs_get_file_info_uri(RVAL2GVFSURI(uri),
                                          info,
                                          options);
@@ -295,7 +296,7 @@ file_m_set_stat(self, uri, info, mask)
 {
   GnomeVFSResult result;
 
-  if (RTEST(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
+  if (RVAL2CBOOL(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
     result = gnome_vfs_set_file_info_uri(RVAL2GVFSURI(uri),
                                          RVAL2GVFSFILEINFO(info),
                                          RVAL2GFLAGS(mask, GNOME_VFS_TYPE_VFS_SET_FILE_INFO_MASK));
@@ -318,7 +319,7 @@ file_m_truncate(self, uri, length)
   /* XXX: how to do this?
    * SafeStringValue(uri);
    */
-  if (RTEST(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
+  if (RVAL2CBOOL(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
     result = gnome_vfs_truncate_uri(RVAL2GVFSURI(uri),
                                     NUM2ULONG(length));
   } else {
@@ -365,8 +366,8 @@ handle_gets(handle, sep, len)
 
         if (len == 1 && c == *sep) {
           break;
-        } else if (memcmp(RSTRING(str)->ptr +
-                          RSTRING(str)->len - len,
+        } else if (memcmp(RSTRING_PTR(str) +
+                          RSTRING_LEN(str) - len,
                           sep, len) == 0) {
           break;
         } else {
@@ -405,11 +406,11 @@ get_gets_separator(argc, argv, sep, len)
 
   if (rb_scan_args(argc, argv, "01", &r_separator) == 1) {
     Check_Type(r_separator, T_STRING);
-    *sep = RSTRING(r_separator)->ptr;
-    *len = RSTRING(r_separator)->len;
+    *sep = RSTRING_PTR(r_separator);
+    *len = RSTRING_LEN(r_separator);
   } else {
-    *sep = RSTRING(rb_rs)->ptr;
-    *len = RSTRING(rb_rs)->len;
+    *sep = RSTRING_PTR(rb_rs);
+    *len = RSTRING_LEN(rb_rs);
   }
 }
 
@@ -438,8 +439,7 @@ file_initialize(argc, argv, self)
     open_mode = OPENMODE2RVAL(GNOME_VFS_OPEN_READ);
   }
 
-  printf ("open_mode = %d\n", RVAL2OPENMODE(open_mode));
-  if (RTEST(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
+  if (RVAL2CBOOL(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
       result = gnome_vfs_open_uri(&handle,
                                   RVAL2GVFSURI(uri),
                                   RVAL2OPENMODE(open_mode));
@@ -448,7 +448,7 @@ file_initialize(argc, argv, self)
         result = gnome_vfs_create_uri(&handle,
                                       RVAL2GVFSURI(uri),
                                       RVAL2OPENMODE(open_mode),
-                                      RTEST(exclusive),
+                                      RVAL2CBOOL(exclusive),
                                       NUM2UINT(perm));
       }
   } else {
@@ -460,7 +460,7 @@ file_initialize(argc, argv, self)
         result = gnome_vfs_create(&handle,
                                   RVAL2CSTR(uri),
                                   RVAL2OPENMODE(open_mode),
-                                  RTEST(exclusive),
+                                  RVAL2CBOOL(exclusive),
                                   NUM2UINT(perm));
       }
   }
@@ -618,12 +618,12 @@ file_write(self, str)
     str = rb_obj_as_string(str);
   }
 
-  if (RSTRING(str)->len == 0) {
+  if (RSTRING_LEN(str) == 0) {
     return INT2FIX(0);
   }
 
-  result = gnome_vfs_write(_SELF(self), RSTRING(str)->ptr,
-                           RSTRING(str)->len, &bytes_written);
+  result = gnome_vfs_write(_SELF(self), RSTRING_PTR(str),
+                           RSTRING_LEN(str), &bytes_written);
   return CHECK_RESULT(result, ULL2NUM(bytes_written));
 }
 
@@ -676,7 +676,7 @@ read_all(handle, siz, str)
     GnomeVFSResult result;
   
     result = gnome_vfs_read(handle,
-                            RSTRING(str)->ptr + bytes,
+                            RSTRING_PTR(str) + bytes,
                             siz - bytes,
                             &bytes_read);
     
@@ -730,11 +730,10 @@ file_read(argc, argv, self)
     return str;
   }
 
-  result = gnome_vfs_read(handle, RSTRING(str)->ptr, bytes,
+  result = gnome_vfs_read(handle, RSTRING_PTR(str), bytes,
                           &bytes_read);
   if (result == GNOME_VFS_OK) {
-    RSTRING(str)->len = bytes_read;
-    RSTRING(str)->ptr[bytes_read] = '\0';
+    rb_str_resize(str, bytes_read);
     return str;
   } else if (result == GNOME_VFS_ERROR_EOF) {
     return Qnil;
@@ -814,7 +813,7 @@ file_seek(argc, argv, self)
   }
 
   return GVFSRESULT2RVAL(gnome_vfs_seek(_SELF(self), whence,
-                                        OFFT2NUM(offset)));
+                                        NUM2ULL(offset)));
 }
 
 static VALUE
