@@ -1,3 +1,4 @@
+/* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
 /*****************************************************************************
  *
  * gnomevfs-monitor.c: GnomeVFS::Monitor class.
@@ -18,9 +19,9 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
  *
- * $Author: sakai $
+ * $Author: ssimons $
  *
- * $Date: 2006/06/13 08:07:33 $
+ * $Date: 2007/07/25 17:08:03 $
  *
  *****************************************************************************/
 
@@ -79,9 +80,9 @@ monitor_callback(handle, monitor_uri, info_uri, event_type, data)
 	GnomeVFSMonitorEventType event_type;
 	gpointer data;
 {
-	rb_funcall((VALUE)data, g_id_call, CSTR2RVAL(monitor_uri),
+	rb_funcall((VALUE)data, g_id_call, 3, CSTR2RVAL(monitor_uri),
 		   CSTR2RVAL(info_uri),
-                   RVAL2GENUM(event_type, GNOME_VFS_TYPE_VFS_MONITOR_EVENT_TYPE));
+                   GENUM2RVAL(event_type, GNOME_VFS_TYPE_VFS_MONITOR_EVENT_TYPE));
 }
 
 static VALUE
@@ -97,7 +98,7 @@ monitor_initialize(argc, argv, self)
 	GnomeVFSResult result;
 
 	rb_scan_args(argc, argv, "2&", &uri, &type, &func);
-	if (RTEST(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
+	if (RVAL2CBOOL(rb_obj_is_kind_of(uri, g_gvfs_uri))) {
 		text_uri = gnome_vfs_uri_to_string(RVAL2GVFSURI(uri),
 						   GNOME_VFS_URI_HIDE_NONE);
 		free_uri = TRUE;
@@ -107,7 +108,7 @@ monitor_initialize(argc, argv, self)
 	}
 
 	if (NIL_P(func)) {
-		func = G_BLOCK_PROC();
+		func = rb_block_proc();
 	}
 	G_RELATIVE(self, func);
 
