@@ -1,5 +1,7 @@
+/* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
 /*
  * Copyright (C) 2003, 2004 Laurent Sansonetti <lrz@gnome.org>
+ * Copyright (C) 2007 Ruby-GNOME2 Project Team
  *
  * This file is part of Ruby/GStreamer.
  *
@@ -29,38 +31,45 @@ static void
 Init_gst_classes (void)
 {
     extern void Init_gst_bin (void);
+    extern void Init_gst_buffer(void);
+    extern void Init_gst_bus(void);
     extern void Init_gst_caps (void);
+    extern void Init_gst_child_proxy(void);
     extern void Init_gst_clock (void);
     extern void Init_gst_clock_entry (void);
-    extern void Init_gst_cpu (void);
     extern void Init_gst_element (void);
     extern void Init_gst_elementfactory (void);
-    extern void Init_gst_event (void);
-    extern void Init_gst_eventmask (void);
-    extern void Init_gst_eventseek (void);
-    extern void Init_gst_eventsegmentseek (void);
-    extern void Init_gst_eventsize (void);
+    extern void Init_gst_event(void);
     extern void Init_gst_format (void);
+    extern void Init_gst_seek(void);
+    extern void Init_gst_structure(void);
     extern void Init_gst_index (void);
     extern void Init_gst_indexentry (void);
     extern void Init_gst_indexfactory (void);
+    extern void Init_gst_message(void);
+    extern void Init_gst_mini_object(void);
     extern void Init_gst_object (void);
     extern void Init_gst_pad (void);
-    extern void Init_gst_padtemplate (void);
+    extern void Init_gst_pad_template(void);
+    extern void Init_gst_ghost_pad(void);
     extern void Init_gst_parse (void);
     extern void Init_gst_pipeline (void);
     extern void Init_gst_plugin (void);
-    extern void Init_gst_pluginfeature (void);
+    extern void Init_gst_plugin_feature (void);
+    extern void Init_gst_query(void);
     extern void Init_gst_querytype (void);
     extern void Init_gst_registry (void);
     extern void Init_gst_registry_pool (void);
     extern void Init_gst_scheduler (void);
     extern void Init_gst_schedulerfactory (void);
+    extern void Init_gst_static_caps(void);
+    extern void Init_gst_static_pad_template(void);
     extern void Init_gst_systemclock (void);
     extern void Init_gst_tag (void);
     extern void Init_gst_tag_setter (void);
     extern void Init_gst_thread (void);
     extern void Init_gst_typefindfactory (void);
+    extern void Init_gst_value(void);
     extern void Init_gst_xml (void);
 
 #if defined(HAVE_GST_OVERLAY)
@@ -72,40 +81,44 @@ Init_gst_classes (void)
 #endif
 
     Init_gst_bin ();
+    Init_gst_bus();
     Init_gst_caps ();
+    Init_gst_child_proxy();
     Init_gst_clock ();
     Init_gst_clock_entry ();
-    Init_gst_cpu ();
     Init_gst_element ();
     Init_gst_elementfactory ();
-    Init_gst_event ();
-    Init_gst_eventmask ();
-    Init_gst_eventseek ();
-    Init_gst_eventsegmentseek ();
-    Init_gst_eventsize ();
+    Init_gst_seek();
     Init_gst_format ();
+    Init_gst_structure();
     Init_gst_index ();
     Init_gst_indexentry ();
     Init_gst_indexfactory ();
     Init_gst_object ();
     Init_gst_pad ();
-    Init_gst_padtemplate ();
+    Init_gst_pad_template();
+    Init_gst_ghost_pad();
     Init_gst_parse ();
     Init_gst_pipeline ();
     Init_gst_plugin ();
-    Init_gst_pluginfeature ();
+    Init_gst_plugin_feature();
     Init_gst_querytype ();
     Init_gst_registry ();
-    Init_gst_registry_pool ();
-    Init_gst_scheduler ();
-    Init_gst_schedulerfactory ();
+    Init_gst_static_caps();
+    Init_gst_static_pad_template();
     Init_gst_systemclock ();
     Init_gst_tag ();
     Init_gst_tag_setter ();
-    Init_gst_thread ();
     Init_gst_typefindfactory ();
+    Init_gst_value();
     Init_gst_xml ();
 
+    Init_gst_mini_object();
+    Init_gst_buffer();
+    Init_gst_message();
+    Init_gst_query();
+    Init_gst_event();
+
 #if defined(HAVE_GST_OVERLAY)
     Init_gst_x_overlay ();
 #endif
@@ -128,7 +141,6 @@ Init_gst_classes (void)
  *
  * Returns: always true, or an exception on failure.
  */
-extern VALUE rb_progname, rb_argv;
 
 static VALUE
 rb_gst_init (int argc, VALUE * argv, VALUE self)
@@ -136,6 +148,12 @@ rb_gst_init (int argc, VALUE * argv, VALUE self)
     gint i, gargc;
     VALUE argary;
     gchar **gargv;
+    GError *error = NULL;
+    static gboolean initialized = FALSE;
+
+    if (initialized)
+        return Qtrue;
+    initialized = TRUE;
 
     /*
      * Convert arguments in a Ruby array.
@@ -144,11 +162,11 @@ rb_gst_init (int argc, VALUE * argv, VALUE self)
     rb_scan_args (argc, argv, "01", &argary);
 
     if (NIL_P (argary)) {
-        gargc = RARRAY (rb_argv)->len;
-        argary = rb_argv;
+        argary = rb_const_get(rb_cObject, rb_intern("ARGV"));
+        gargc = RARRAY_LEN(argary);
     } else {
-        Check_Type (argary, T_ARRAY);
-        gargc = RARRAY (argary)->len;
+        Check_Type(argary, T_ARRAY);
+        gargc = RARRAY_LEN(argary);
     }
 
     /*
@@ -156,11 +174,11 @@ rb_gst_init (int argc, VALUE * argv, VALUE self)
      */
     gargv = ALLOCA_N (char *, gargc + 1);
 
-    gargv[0] = (gchar *) STR2CSTR (rb_progname);
+    gargv[0] = RVAL2CSTR(rb_argv0);
 
     for (i = 0; i < gargc; i++) {
-        if (TYPE (RARRAY (argary)->ptr[i]) == T_STRING) {
-            gargv[i + 1] = (gchar *) STR2CSTR (RARRAY (argary)->ptr[i]);
+        if (TYPE (RARRAY_PTR(argary)[i]) == T_STRING) {
+            gargv[i + 1] = RVAL2CSTR(RARRAY_PTR(argary)[i]);
         } else {
             gargv[i + 1] = "";
         }
@@ -168,16 +186,18 @@ rb_gst_init (int argc, VALUE * argv, VALUE self)
     gargc++;
 
     /*
-     * Now, we can initialize GStreamer. 
+     * Now, we can initialize GStreamer.
      */
-    if (gst_init_check (&gargc, &gargv) == FALSE)
-        rb_raise (rb_eRuntimeError, "failed to initialize GStreamer");
+    if (!gst_init_check(&gargc, &gargv, &error))
+        RAISE_GERROR(error);
 
     /*
      * Initialize all included classes 
      */
     Init_gst_classes ();
 
+    rbgutil_start_callback_dispatch_thread();
+
     return Qtrue;
 }
 
@@ -187,106 +207,62 @@ rb_gst_init (int argc, VALUE * argv, VALUE self)
  * Gets the version number of the GStreamer library, in an array
  * of 3 fixnums, which represent major, minor and macro numbers. 
  *
- *	# Prints GStreamer version in a String 'major.minor.macro'
+ *	# Prints GStreamer version in a String 'major.minor.macro.nano'
  *	p Gst.version.join('.')
  *
- * Returns: an Array of 3 fixnums (major, minor and macro numbers).
+ * Returns: an Array of 4 fixnums (major, minor, macro and nano numbers).
  */
 static VALUE
 rb_gst_version (VALUE self)
 {
-    guint major, minor, micro;
-    VALUE arr;
+    guint major, minor, micro, nano;
 
-    gst_version (&major, &minor, &micro);
-    arr = rb_ary_new ();
-    rb_ary_push (arr, INT2FIX (major));
-    rb_ary_push (arr, INT2FIX (minor));
-    rb_ary_push (arr, INT2FIX (micro));
-
-    return arr;
+    gst_version(&major, &minor, &micro, &nano);
+    return rb_ary_new3(4,
+                       INT2NUM(major), INT2NUM(minor),
+                       INT2NUM(micro), INT2NUM(nano));
 }
 
-/* Class method: has_threads?
- * Returns: true if GStreamer has threads enabled, false otherwise.
- */
 static VALUE
-rb_gst_has_threads (VALUE self)
+rbgst_m_check_version(VALUE self, VALUE rb_major, VALUE rb_minor, VALUE rb_micro)
 {
-    return CBOOL2RVAL (gst_has_threads ());
+    guint major, minor, micro, nano;
+
+    gst_version(&major, &minor, &micro, &nano);
+    return CBOOL2RVAL(major > NUM2UINT(rb_major) ||
+                      (major == NUM2UINT(rb_major) &&
+                       minor > NUM2UINT(rb_minor)) ||
+                      (major == NUM2UINT(rb_major) &&
+                       minor == NUM2UINT(rb_minor) &&
+                       micro >= NUM2UINT(rb_micro)));
 }
 
-/*
- * Class method: use_threads(state)
- * state: a boolean state.
- *
- * Instructs the core to turn on/off threading. 
- *
- * When threading is turned off, all thread operations such as 
- * mutexes and conditionals are turned into NOPs. 
- * Use this if you want absolute minimal overhead and you don't 
- * use any threads in the pipeline.
- *
- * Returns: the provided boolean value.
- */
-static VALUE
-rb_gst_use_threads (VALUE self, VALUE value)
-{
-    gst_use_threads (RVAL2CBOOL (value));
-    return value;
-}
 
 void
 Init_gst (void)
 {
     mGst = rb_define_module ("Gst");
 
-    rb_define_module_function (mGst, "init", rb_gst_init, -1);
-    rb_define_module_function (mGst, "version", rb_gst_version, 0);
-    rb_define_module_function (mGst, "has_threads?", rb_gst_has_threads, 0);
-    rb_define_module_function (mGst, "use_threads", rb_gst_use_threads, 1);
-
-    /*
-     * Constant: SECOND
-     * Constant that defines one GStreamer second.
-     */
-    rb_define_const (mGst, "SECOND", INT2FIX (GST_SECOND));
-
-    /*
-     * Constant: MSECOND
-     * Constant that defines one GStreamer millisecond.
-     */
-    rb_define_const (mGst, "MSECOND", INT2FIX (GST_MSECOND));
-
-    /*
-     * Constant: USECOND
-     * Constant that defines one GStreamer microsecond.
-     */
-    rb_define_const (mGst, "USECOND", INT2FIX (GST_USECOND));
-
-    /*
-     * Constant: NSECOND
-     * Constant that defines one GStreamer nanosecond.
-     */
-    rb_define_const (mGst, "NSECOND", INT2FIX (GST_NSECOND));
+    rb_define_module_function(mGst, "init", rb_gst_init, -1);
+    rb_define_module_function(mGst, "version", rb_gst_version, 0);
 
     /*
      * Constant: VERSION_MAJOR
      * GStreamer's major version.
      */
-    rb_define_const (mGst, "VERSION_MAJOR", INT2FIX (GST_VERSION_MAJOR));
+    rb_define_const(mGst, "VERSION_MAJOR", INT2FIX(GST_VERSION_MAJOR));
 
     /*
      * Constant: VERSION_MINOR
      * GStreamer's minor version.
      */
-    rb_define_const (mGst, "VERSION_MINOR", INT2FIX (GST_VERSION_MINOR));
+    rb_define_const(mGst, "VERSION_MINOR", INT2FIX(GST_VERSION_MINOR));
 
     /*
      * Constant: VERSION_MICRO
      * GStreamer's micro version.
      */
-    rb_define_const (mGst, "VERSION_MICRO", INT2FIX (GST_VERSION_MICRO));
+    rb_define_const(mGst, "VERSION_MICRO", INT2FIX(GST_VERSION_MICRO));
 
 
     /*
@@ -298,4 +274,6 @@ Init_gst (void)
                                 INT2FIX(GST_VERSION_MAJOR),
                                 INT2FIX(GST_VERSION_MINOR),
                                 INT2FIX(GST_VERSION_MICRO)));
+
+    rb_define_module_function(mGst, "check_version?", rbgst_m_check_version, 3);
 }
