@@ -3,8 +3,8 @@
 
   rbgdkatom.c -
 
-  $Author: mutoh $
-  $Date: 2005/07/30 11:22:15 $
+  $Author: sakai $
+  $Date: 2007/07/15 15:03:11 $
 
   Copyright (C) 2002,2003 Masao Mutoh
 ************************************************/
@@ -57,10 +57,14 @@ gdkatom_s_intern(argc, argv, self)
   
     rb_scan_args(argc, argv, "11", &name, &exist);
 
-    return BOXED2RVAL(gdk_atom_intern(RVAL2CSTR(name), RTEST(exist)), 
+    return BOXED2RVAL(gdk_atom_intern(RVAL2CSTR(name), RVAL2CBOOL(exist)), 
                       GDK_TYPE_ATOM);
 }
 
+/* We don't need them.
+GdkAtom gdk_atom_intern_static_string(const gchar *atom_name);
+ */
+
 static VALUE
 gdkatom_initialize(self, num)
     VALUE self, num;
@@ -95,7 +99,7 @@ static VALUE
 gdkatom_eq(self, other)
     VALUE self, other;
 {
-    return (_SELF(self) == _SELF(other)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(_SELF(self) == _SELF(other));
 }
 
 void
