@@ -4,7 +4,7 @@
   rbgdkevent.c -
 
   $Author: ggc $
-  $Date: 2006/06/22 19:52:54 $
+  $Date: 2007/07/13 16:07:31 $
 
   Copyright (C) 2002-2004 Ruby-GNOME2 Project Team
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
@@ -115,7 +115,13 @@ static VALUE \
 gdkevent ## type ## _set_ ## name (self, val)\
     VALUE self, val;\
 {\
-    get_gdkevent(self)->type.name = RVAL2GOBJ(val);\
+    GdkEvent *event;\
+    event = get_gdkevent(self);\
+    if (event->type.name)\
+      g_object_unref(event->type.name);\
+    event->type.name = RVAL2GOBJ(val);\
+    if (event->type.name)\
+      g_object_ref(event->type.name);\
     return self;\
 }
 
@@ -130,7 +136,7 @@ static VALUE \
 gdkevent ## type ## _set_ ## name (self, val)\
     VALUE self, val;\
 {\
-    get_gdkevent(self)->type.name = RTEST(val);\
+    get_gdkevent(self)->type.name = RVAL2CBOOL(val);\
     return self;\
 }
 
@@ -250,7 +256,7 @@ static VALUE
 gdkevent_s_events_pending(self)
     VALUE self;
 {
-    return gdk_events_pending() ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gdk_events_pending());
 }
 
 static VALUE
@@ -352,7 +358,7 @@ static VALUE
 gdkevent_s_handler_set(self)
     VALUE self;
 {
-    volatile VALUE func = G_BLOCK_PROC();
+    volatile VALUE func = rb_block_proc();
     G_RELATIVE(self, func);
 
     gdk_event_handler_set((GdkEventFunc)handler_func, (gpointer)func, NULL);
@@ -370,7 +376,7 @@ static VALUE
 gdkevent_s_set_show_events(self, show_events)
     VALUE self, show_events;
 {
-    gdk_set_show_events(RTEST(show_events));
+    gdk_set_show_events(RVAL2CBOOL(show_events));
     return self;
 }
 #if GTK_CHECK_VERSION(2,2,0)
@@ -464,6 +470,15 @@ ATTR_BOOL(motion, is_hint);
 ATTR_GOBJ(motion, device);
 GDKEVENT_INIT(motion, GDK_MOTION_NOTIFY);
 
+#if GTK_CHECK_VERSION(2, 12, 0)
+static VALUE
+gdkeventmotion_request_motions(VALUE self)
+{
+    gdk_event_request_motions(&(get_gdkevent(self)->motion));
+    return self;
+}
+#endif
+
 /* GdkEventExpose */
 static VALUE
 gdkeventexpose_area(self)
@@ -627,7 +642,7 @@ filter_func(xevent, event, func)
     GdkEvent* event;
     gpointer func;
 {
-    VALUE ret = rb_funcall((VALUE)func, id_call, 2, UINT2NUM((guint)xevent), make_gdkevent(event));
+    VALUE ret = rb_funcall((VALUE)func, id_call, 2, LONG2NUM((glong)xevent), make_gdkevent(event));
     return RVAL2GENUM(ret, GDK_TYPE_FILTER_RETURN);
 }
 
@@ -635,7 +650,7 @@ static VALUE
 gdkevent_s_add_client_message_filter(self, message_type)
     VALUE self, message_type;
 {
-    volatile VALUE func = G_BLOCK_PROC();
+    volatile VALUE func = rb_block_proc();
     G_RELATIVE(self, func);
     gdk_add_client_message_filter(RVAL2ATOM(message_type),
                                   (GdkFilterFunc)filter_func, (gpointer)func);
@@ -816,6 +831,9 @@ Init_gtk_gdk_event()
     DEFINE_ACCESSOR(ev, motion, x_root);
     DEFINE_ACCESSOR(ev, motion, y_root);
     DEFINE_INIT(ev, motion);
+#if GTK_CHECK_VERSION(2, 12, 0)
+    rb_define_method(ev, "request", gdkeventmotion_request_motions, 0);
+#endif
     G_DEF_SETTERS(ev);
 
     /* GdkEventExpose */
