@@ -3,8 +3,8 @@
 
   rbgdkgc.c -
 
-  $Author: mutoh $
-  $Date: 2005/07/22 18:07:01 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:31 $
 
   Copyright (C) 2002-2004 Ruby-GNOME2 Project Team
   Copyright (C) 2002,2003 Masao Mutoh
@@ -15,6 +15,9 @@
 
 #define _SELF(s) (GDK_GC(RVAL2GOBJ(s)))
 
+static VALUE gdkGC;
+static VALUE gdkDrawable;
+
 static VALUE
 gdkgc_initialize(self, win)
     VALUE self, win;
@@ -38,8 +41,7 @@ static VALUE
 gdkgc_set_foreground(self, color)
     VALUE self, color;
 {
-    gdk_gc_set_foreground(_SELF(self), 
-                          (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+    gdk_gc_set_foreground(_SELF(self), RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -47,8 +49,7 @@ static VALUE
 gdkgc_set_background(self, color)
     VALUE self, color;
 {
-    gdk_gc_set_background(_SELF(self), 
-                          (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+    gdk_gc_set_background(_SELF(self), RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -56,8 +57,7 @@ static VALUE
 gdkgc_set_rgb_fg_color(self, color)
     VALUE self, color;
 {
-    gdk_gc_set_rgb_fg_color(_SELF(self), 
-                            (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+    gdk_gc_set_rgb_fg_color(_SELF(self), RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -65,8 +65,7 @@ static VALUE
 gdkgc_set_rgb_bg_color(self, color)
     VALUE self, color;
 {
-    gdk_gc_set_rgb_bg_color(_SELF(self), 
-                            (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+    gdk_gc_set_rgb_bg_color(_SELF(self), RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -161,7 +160,7 @@ static VALUE
 gdkgc_set_exposures(self, exposures)
     VALUE self, exposures;
 {
-    gdk_gc_set_exposures(_SELF(self), RTEST(exposures));
+    gdk_gc_set_exposures(_SELF(self), RVAL2CBOOL(exposures));
     return self;
 }
 
@@ -185,24 +184,28 @@ gdkgc_set_dashes(self, dash_offset, dash_list)
 
     Check_Type(dash_list, T_ARRAY);
 
-    buf = ALLOCA_N(gint8, RARRAY(dash_list)->len);
-    for (i = 0; i < RARRAY(dash_list)->len; i++) {
-        Check_Type(RARRAY(dash_list)->ptr[i], T_FIXNUM);
-        buf[i] = (gint8)NUM2CHR(RARRAY(dash_list)->ptr[i]);
+    buf = ALLOCA_N(gint8, RARRAY_LEN(dash_list));
+    for (i = 0; i < RARRAY_LEN(dash_list); i++) {
+        Check_Type(RARRAY_PTR(dash_list)[i], T_FIXNUM);
+        buf[i] = (gint8)NUM2CHR(RARRAY_PTR(dash_list)[i]);
     }
     gdk_gc_set_dashes(_SELF(self), NUM2INT(dash_offset),
-                      (gint8*)buf, RARRAY(dash_list)->len);
+                      (gint8*)buf, RARRAY_LEN(dash_list));
 
     return self;
 }
 
 static VALUE
-gdkgc_copy(self)
-    VALUE self;
+gdkgc_copy(self, dst)
+    VALUE self, dst;
 {
-    GdkGC copy;
-    gdk_gc_copy(&copy, _SELF(self));
-    return GOBJ2RVAL(&copy);
+    if (RVAL2CBOOL(rb_obj_is_kind_of(dst, gdkDrawable))) {
+        VALUE args[1];
+        args[0] = dst;
+        dst = rb_class_new_instance(1, args, gdkGC);
+    }
+    gdk_gc_copy(_SELF(dst), _SELF(self));
+    return dst;
 }
 
 static VALUE
@@ -365,8 +368,9 @@ gdkgc_screen(self)
 void
 Init_gtk_gdk_gc()
 {
-    VALUE gdkGC = G_DEF_CLASS(GDK_TYPE_GC, "GC", mGdk);
-
+    gdkGC = G_DEF_CLASS(GDK_TYPE_GC, "GC", mGdk);
+    gdkDrawable = rb_const_get(mGdk, rb_intern("Drawable"));
+        
     rbgobj_add_abstract_but_create_instance_class(GDK_TYPE_GC);
 
     rb_define_method(gdkGC, "initialize", gdkgc_initialize, 1);
@@ -387,7 +391,7 @@ Init_gtk_gdk_gc()
     rb_define_method(gdkGC, "set_exposures", gdkgc_set_exposures, 1);
     rb_define_method(gdkGC, "set_line_attributes", gdkgc_set_line_attributes, 4);
     rb_define_method(gdkGC, "set_dashes", gdkgc_set_dashes, 2);
-    rb_define_method(gdkGC, "copy", gdkgc_copy, 0);
+    rb_define_method(gdkGC, "copy", gdkgc_copy, 1);
     rb_define_method(gdkGC, "set_colormap", gdkgc_set_colormap, 1);
 
     rb_define_method(gdkGC, "foreground", gdkgc_get_foreground, 0);
