@@ -3,8 +3,8 @@
 
   rbgdkrgb.c -
 
-  $Author: mutoh $
-  $Date: 2005/07/22 18:07:18 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:31 $
 
   Copyright (C) 2002,2003 Ruby-GNOME2 Project Team
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
@@ -56,14 +56,14 @@ rgb_draw_indexed_image(self, win, gc, x, y, w, h, dither, buf, rowstride, colors
     guint32* gcolors;
     gint i, n_colors;
 
-    n_colors = RARRAY(colors)->len;
+    n_colors = RARRAY_LEN(colors);
 
     if (n_colors > 255)
         rb_raise(rb_eArgError, "colors: out of range (0 - 255)");
 
     gcolors = g_new(guint32, n_colors);
     for (i = 0; i < n_colors; i++) {
-        gcolors[i] = NUM2UINT(RARRAY(colors)->ptr[i]);
+        gcolors[i] = NUM2UINT(RARRAY_PTR(colors)[i]);
     }
 
     cmap = gdk_rgb_cmap_new(gcolors, n_colors);
@@ -125,7 +125,7 @@ rgb_find_color(self, colormap, color)
     VALUE self, colormap, color;
 {
     gdk_rgb_find_color(GDK_COLORMAP(RVAL2GOBJ(colormap)),
-                       (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+                       RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -133,7 +133,7 @@ static VALUE
 rgb_set_install(self, install)
     VALUE self, install;
 {
-    gdk_rgb_set_install(RTEST(install));
+    gdk_rgb_set_install(RVAL2CBOOL(install));
     return self;
 }
 
@@ -170,7 +170,7 @@ static VALUE
 rgb_set_verbose(self, verbose)
     VALUE self, verbose;
 {
-    gdk_rgb_set_verbose(RTEST(verbose));
+    gdk_rgb_set_verbose(RVAL2CBOOL(verbose));
     return self;
 }
 
