@@ -3,8 +3,8 @@
 
   rbgdkwindow.c -
 
-  $Author: mutoh $
-  $Date: 2006/11/25 17:50:41 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:31 $
 
   Copyright (C) 2002-2006 Ruby-GNOME2 Project Team
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
@@ -83,14 +83,14 @@ static VALUE
 gdkwin_is_visible(self)
     VALUE self;
 {
-    return gdk_window_is_visible(_SELF(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gdk_window_is_visible(_SELF(self)));
 }
 
 static VALUE
 gdkwin_is_viewable(self)
     VALUE self;
 {
-    return gdk_window_is_viewable(_SELF(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gdk_window_is_viewable(_SELF(self)));
 }
 
 static VALUE
@@ -177,7 +177,7 @@ static VALUE
 gdkwin_set_keep_above(self, setting)
     VALUE self, setting;
 {
-    gdk_window_set_keep_above(_SELF(self), RTEST(setting));
+    gdk_window_set_keep_above(_SELF(self), RVAL2CBOOL(setting));
     return self;
 }
 
@@ -185,7 +185,7 @@ static VALUE
 gdkwin_set_keep_below(self, setting)
     VALUE self, setting;
 {
-    gdk_window_set_keep_below(_SELF(self), RTEST(setting));
+    gdk_window_set_keep_below(_SELF(self), RVAL2CBOOL(setting));
     return self;
 }
 #endif
@@ -260,7 +260,7 @@ gdkwin_clear_area(argc, argv, self)
     VALUE gen_expose, x, y, w, h;
     rb_scan_args(argc, argv, "41", &x, &y, &w, &h, &gen_expose);
 
-    if (! NIL_P(gen_expose) && RTEST(gen_expose)){
+    if (! NIL_P(gen_expose) && RVAL2CBOOL(gen_expose)){
         gdk_window_clear_area_e(_SELF(self),
                                 NUM2INT(x), NUM2INT(y), NUM2INT(w), NUM2INT(h));
     } else {
@@ -339,6 +339,15 @@ gdkwin_s_constrain_size(self, geometry, flags, w, h)
     return rb_assoc_new(INT2NUM(new_width), INT2NUM(new_height));
 }
 
+#if GTK_CHECK_VERSION(2, 12, 0)
+static VALUE
+gdkwin_beep(VALUE self) 
+{
+    gdk_window_beep(_SELF(self));
+    return self;
+}
+#endif
+
 static VALUE
 gdkwin_begin_paint(self, area)
     VALUE self, area;
@@ -368,11 +377,11 @@ gdkwin_invalidate(self, area, invalidate_children)
     if (rb_obj_is_kind_of(area, GTYPE2CLASS(GDK_TYPE_RECTANGLE))){
         gdk_window_invalidate_rect(_SELF(self),
                                    RVAL2BOXED(area, GDK_TYPE_RECTANGLE),
-                                   RTEST(invalidate_children));
+                                   RVAL2CBOOL(invalidate_children));
     } else {
         gdk_window_invalidate_region(_SELF(self),
                                      RVAL2BOXED(area, GDK_TYPE_REGION),
-                                     RTEST(invalidate_children));
+                                     RVAL2CBOOL(invalidate_children));
     }
     return self;
 }
@@ -383,7 +392,7 @@ invalidate_child_func_wrap(window, func)
     VALUE func;
 {
     VALUE result = rb_funcall(func, id_call, 1, GOBJ2RVAL(window));
-    return RTEST(result);
+    return RVAL2CBOOL(result);
 }
 
 typedef gboolean (*ChildFunc) (GdkWindow*, gpointer);
@@ -394,7 +403,7 @@ gdkwin_invalidate_maybe_recurse(self, region)
 {
     VALUE func = (VALUE)NULL;
     if (rb_block_given_p()){
-        func = G_BLOCK_PROC();
+        func = rb_block_proc();
         G_RELATIVE(self, func);
     }
     gdk_window_invalidate_maybe_recurse(_SELF(self),
@@ -440,7 +449,7 @@ static VALUE
 gdkwin_process_updates(self, update_children)
     VALUE self, update_children;
 {
-    gdk_window_process_updates(_SELF(self), RTEST(update_children));
+    gdk_window_process_updates(_SELF(self), RVAL2CBOOL(update_children));
     return self;
 }
 
@@ -448,7 +457,7 @@ static VALUE
 gdkwin_s_set_debug_updates(self, setting)
     VALUE self, setting;
 {
-    gdk_window_set_debug_updates(RTEST(setting));
+    gdk_window_set_debug_updates(RVAL2CBOOL(setting));
     return self;
 }
 
@@ -500,7 +509,7 @@ gdkwin_set_override_redirect(self, override_redirect)
     VALUE self, override_redirect;
 {
     gdk_window_set_override_redirect(_SELF(self), 
-                                     RTEST(override_redirect));
+                                     RVAL2CBOOL(override_redirect));
     return self;
 }
 
@@ -509,7 +518,7 @@ static VALUE
 gdkwin_set_accept_focus(self, accept_focus)
     VALUE self, accept_focus;
 {
-    gdk_window_set_accept_focus(_SELF(self), RTEST(accept_focus));
+    gdk_window_set_accept_focus(_SELF(self), RVAL2CBOOL(accept_focus));
     return self;
 }
 #endif
@@ -519,7 +528,7 @@ static VALUE
 gdkwin_set_focus_on_map(self, focus_on_map)
     VALUE self, focus_on_map;
 {
-    gdk_window_set_focus_on_map(_SELF(self), RTEST(focus_on_map));
+    gdk_window_set_focus_on_map(_SELF(self), RVAL2CBOOL(focus_on_map));
     return self;
 }
 #endif
@@ -617,7 +626,7 @@ gdkwin_set_static_gravities(self, use_static)
     VALUE self, use_static;
 {
     gboolean ret = gdk_window_set_static_gravities(_SELF(self),
-                                                   RTEST(use_static));
+                                                   RVAL2CBOOL(use_static));
     if (! ret)
         rb_raise(rb_eRuntimeError, "couldn't turn on static gravity");
 
@@ -640,8 +649,7 @@ static VALUE
 gdkwin_set_background(self, color)
     VALUE self, color;
 {
-    gdk_window_set_background(_SELF(self), 
-                              (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+    gdk_window_set_background(_SELF(self), RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -651,7 +659,7 @@ gdkwin_set_back_pixmap(self, pixmap, parent_relative)
 {
     gdk_window_set_back_pixmap(_SELF(self), 
                                GDK_PIXMAP(RVAL2GOBJ(pixmap)),
-                               RTEST(parent_relative));
+                               RVAL2CBOOL(parent_relative));
     return self;
 }
 
@@ -706,8 +714,8 @@ gdkwin_set_icon_list(self, pixbufs)
     GList *glist = NULL;
 
     Check_Type(pixbufs, T_ARRAY);
-    for (i = 0; i < RARRAY(pixbufs)->len; i++) {
-        glist = g_list_append(glist, GDK_PIXBUF(RVAL2GOBJ(RARRAY(pixbufs)->ptr[i])));
+    for (i = 0; i < RARRAY_LEN(pixbufs); i++) {
+        glist = g_list_append(glist, GDK_PIXBUF(RVAL2GOBJ(RARRAY_PTR(pixbufs)[i])));
     }
 
     gdk_window_set_icon_list(_SELF(self), glist);
@@ -718,7 +726,7 @@ static VALUE
 gdkwin_set_modal_hint(self, modal)
     VALUE self, modal;
 {
-    gdk_window_set_modal_hint(_SELF(self), RTEST(modal));
+    gdk_window_set_modal_hint(_SELF(self), RVAL2CBOOL(modal));
     return self;
 }
 
@@ -744,7 +752,7 @@ static VALUE
 gdkwin_set_skip_taskbar_hint(self, hint)
     VALUE self, hint;
 {
-    gdk_window_set_skip_taskbar_hint(_SELF(self), RTEST(hint));
+    gdk_window_set_skip_taskbar_hint(_SELF(self), RVAL2CBOOL(hint));
     return self;
 }
 
@@ -752,7 +760,7 @@ static VALUE
 gdkwin_set_skip_pager_hint(self, hint)
     VALUE self, hint;
 {
-    gdk_window_set_skip_pager_hint(_SELF(self), RTEST(hint));
+    gdk_window_set_skip_pager_hint(_SELF(self), RVAL2CBOOL(hint));
     return self;
 }
 #endif
@@ -762,7 +770,7 @@ static VALUE
 gdkwin_set_urgency_hint(self, hint)
     VALUE self, hint;
 {
-    gdk_window_set_urgency_hint(_SELF(self), RTEST(hint));
+    gdk_window_set_urgency_hint(_SELF(self), RVAL2CBOOL(hint));
     return self;
 }
 #endif
@@ -946,6 +954,29 @@ gdkwin_set_functions(self, func)
     return self;
 }
 
+#if GTK_CHECK_VERSION(2, 12, 0)
+static VALUE
+gdkwin_set_composited(VALUE self, VALUE composited)
+{
+    gdk_window_set_composited(_SELF(self), RVAL2CBOOL(composited));
+    return self;
+}
+
+static VALUE
+gdkwin_set_opacity(VALUE self, VALUE opacity)
+{
+    gdk_window_set_opacity(_SELF(self), NUM2DBL(opacity));
+    return self;
+}
+
+static VALUE
+gdkwin_set_startup_id(VALUE self, VALUE startup_id)
+{
+    gdk_window_set_startup_id(_SELF(self), RVAL2CSTR_ACCEPT_NIL(startup_id));
+    return self;
+}
+#endif
+
 static VALUE
 gdkwin_s_get_toplevels(self)
     VALUE self;
@@ -1038,6 +1069,15 @@ gdkwin_get_server_time(self)
     return UINT2NUM(gdk_x11_get_server_time(_SELF(self)));
 }
 
+#if GTK_CHECK_VERSION(2,6,0)
+static VALUE
+gdkwin_set_user_time(VALUE self, VALUE time)
+{
+    gdk_x11_window_set_user_time(_SELF(self), NUM2UINT(time));
+    return Qnil;
+}
+#endif
+
 #if GTK_CHECK_VERSION(2,8,0)
 static VALUE
 gdkwin_move_to_current_desktop(self)
@@ -1097,6 +1137,9 @@ Init_gtk_gdk_window()
     rb_define_method(gdkWindow, "lower", gdkwin_lower, 0);
     rb_define_method(gdkWindow, "focus", gdkwin_focus, 1);
     rb_define_method(gdkWindow, "register_dnd", gdkwin_register_dnd, 0);
+#if GTK_CHECK_VERSION(2, 12, 0)
+    rb_define_method(gdkWindow, "beep", gdkwin_beep, 0);
+#endif
     rb_define_method(gdkWindow, "begin_resize_drag", gdkwin_begin_resize_drag, 5);
     rb_define_method(gdkWindow, "begin_move_drag", gdkwin_begin_move_drag, 4);
     rb_define_method(gdkWindow, "begin_paint", gdkwin_begin_paint, 1);
@@ -1174,6 +1217,11 @@ Init_gtk_gdk_window()
     rb_define_method(gdkWindow, "set_decorations", gdkwin_set_decorations, 1);
     rb_define_method(gdkWindow, "decorations", gdkwin_get_decorations, 0);
     rb_define_method(gdkWindow, "set_functions", gdkwin_set_functions, 1);
+#if GTK_CHECK_VERSION(2, 12, 0)
+    rb_define_method(gdkWindow, "set_composited", gdkwin_set_composited, 1);
+    rb_define_method(gdkWindow, "set_opacity", gdkwin_set_opacity, 1);
+    rb_define_method(gdkWindow, "set_startup_id", gdkwin_set_startup_id, 1);
+#endif
     rb_define_singleton_method(gdkWindow, "toplevels", gdkwin_s_get_toplevels, 0);
 
     rb_define_singleton_method(gdkWindow, "foreign_new", gdkwin_foreign_new, -1);
@@ -1230,6 +1278,9 @@ Init_gtk_gdk_window()
 #ifdef GDK_WINDOWING_X11
     rb_define_method(gdkWindow, "server_time", gdkwin_get_server_time, 0);
 
+#if GTK_CHECK_VERSION(2,6,0)
+    rb_define_method(gdkWindow, "set_user_time", gdkwin_set_user_time, 1);
+#endif
 #if GTK_CHECK_VERSION(2,8,0)
     rb_define_method(gdkWindow, "move_to_current_desktop", gdkwin_move_to_current_desktop, 0);
 #endif
