@@ -3,8 +3,8 @@
 
   rbgtkcelllayout.c -
 
-  $Author: mutoh $
-  $Date: 2005/01/29 11:44:14 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:31 $
 
   Copyright (C) 2004 Masao Mutoh
 ************************************************/
@@ -21,7 +21,7 @@ layout_pack_start(self, cell, expand)
     VALUE self, cell, expand;
 {
     gtk_cell_layout_pack_start(_SELF(self), RVAL2RENDERER(cell),
-                               RTEST(expand));
+                               RVAL2CBOOL(expand));
     return self;
 }
 
@@ -30,7 +30,7 @@ layout_pack_end(self, cell, expand)
     VALUE self, cell, expand;
 {
     gtk_cell_layout_pack_end(_SELF(self), RVAL2RENDERER(cell),
-                             RTEST(expand));
+                             RVAL2CBOOL(expand));
     return self;
 }
 
@@ -55,7 +55,7 @@ static VALUE
 layout_add_attribute(self, cell, attribute, column)
     VALUE self, cell, attribute, column;
 {
-    gchar* name;
+    const gchar *name;
 
     if (SYMBOL_P(attribute)) {
         name = rb_id2name(SYM2ID(attribute));
@@ -78,7 +78,7 @@ layout_data_func(layout, cell, tree_model, iter, func)
 {
     iter->user_data3 = tree_model;
     rb_funcall((VALUE)func, id_call, 4, GOBJ2RVAL(layout), GOBJ2RVAL(cell),
-               GOBJ2RVAL(tree_model), BOXED2RVAL(iter, GTK_TYPE_TREE_ITER));
+               GOBJ2RVAL(tree_model), GTKTREEITER2RVAL(iter));
 }
 
 static VALUE
@@ -86,7 +86,7 @@ layout_set_cell_data_func(self, cell)
     VALUE self, cell;
 {
     if (rb_block_given_p()) {
-        VALUE func = G_BLOCK_PROC();
+        VALUE func = rb_block_proc();
         G_RELATIVE(self, func);
         gtk_cell_layout_set_cell_data_func(_SELF(self), RVAL2RENDERER(cell),
                                            (GtkCellLayoutDataFunc)layout_data_func, 
@@ -117,10 +117,10 @@ layout_set_attributes(self, cell, attrs)
     layout_clear_attributes(self, cell);
 
     ary = rb_funcall(attrs, rb_intern("to_a"), 0);
-    for (i = 0; i < RARRAY(ary)->len; i++){
+    for (i = 0; i < RARRAY_LEN(ary); i++){
         layout_add_attribute(self, cell, 
-                             RARRAY(RARRAY(ary)->ptr[i])->ptr[0],
-                             RARRAY(RARRAY(ary)->ptr[i])->ptr[1]);
+                             RARRAY_PTR(RARRAY_PTR(ary)[i])[0],
+                             RARRAY_PTR(RARRAY_PTR(ary)[i])[1]);
     }
     return self;
 }
