@@ -3,10 +3,10 @@
 
   rbgtkcontainer.c -
 
-  $Author: mutoh $
-  $Date: 2005/07/22 18:07:18 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:31 $
 
-  Copyright (C) 2002-2004 Ruby-GNOME2 Project Team
+  Copyright (C) 2002-2006 Ruby-GNOME2 Project Team
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
                           Daisuke Kanda,
                           Hiroshi Igarashi
@@ -28,13 +28,14 @@ static VALUE
 cont_is_resize_container(self)
     VALUE self;
 {
-    return GTK_IS_RESIZE_CONTAINER(_SELF(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(GTK_IS_RESIZE_CONTAINER(_SELF(self)));
 }
 
 static VALUE
 cont_remove(self, other)
     VALUE self, other;
 {
+    G_CHILD_REMOVE(self, other);
     gtk_container_remove(_SELF(self), GTK_WIDGET(RVAL2GOBJ(other)));
     return self;
 }
@@ -57,7 +58,7 @@ cont_foreach(argc, argv, self)
 
     rb_scan_args(argc, argv, "01", &callback);
     if (NIL_P(callback)) {
-        callback = G_BLOCK_PROC();
+        callback = rb_block_proc();
     }
     gtk_container_foreach(_SELF(self), exec_callback, (gpointer)callback);
     return self;
@@ -73,7 +74,7 @@ cont_forall(argc, argv, self)
 
     rb_scan_args(argc, argv, "01", &callback);
     if (NIL_P(callback)) {
-        callback = G_BLOCK_PROC();
+        callback = rb_block_proc();
     }
     gtk_container_forall(_SELF(self), exec_callback, (gpointer)callback);
     return self;
@@ -83,14 +84,14 @@ static VALUE
 cont_get_children(self)
     VALUE self;
 {
-   return GLIST2ARY(gtk_container_get_children(_SELF(self)));
+   return GLIST2ARYF(gtk_container_get_children(_SELF(self)));
 }
 
 static VALUE
 cont_set_reallocate_redraws(self, needs_redraws)
     VALUE self, needs_redraws;
 {
-    gtk_container_set_reallocate_redraws(_SELF(self), RTEST(needs_redraws));
+    gtk_container_set_reallocate_redraws(_SELF(self), RVAL2CBOOL(needs_redraws));
     return self;
 }
 
@@ -247,6 +248,9 @@ cont_child_get_property(self, child, prop_name)
                                          GTK_WIDGET(RVAL2GOBJ(child)),
                                          name , &gval);
         ret = getter ? getter(&gval) : GVAL2RVAL(&gval);
+
+        G_CHILD_ADD(child, ret);
+
         g_value_unset(&gval);
         return ret;
     }
@@ -293,6 +297,8 @@ cont_child_set_property(self, child, prop_name, val)
                 rbgobj_rvalue_to_gvalue(val, &gval);
         }
 
+        G_CHILD_ADD(child, val);
+
         gtk_container_child_set_property(GTK_CONTAINER(RVAL2GOBJ(self)), 
                                          GTK_WIDGET(RVAL2GOBJ(child)), name, &gval);
 
@@ -315,6 +321,9 @@ cont_add(argc, argv, self)
     child = GTK_WIDGET(RVAL2GOBJ(other));
     gtk_widget_freeze_child_notify(child);
     gtk_container_add(_SELF(self), child);
+
+    G_CHILD_ADD(self, other);
+
     if (child->parent && (! NIL_P(properties))){
         int i;
         VALUE ary;
@@ -324,10 +333,10 @@ cont_add(argc, argv, self)
         ary = rb_funcall(properties, rb_intern("to_a"), 0);
         obj = RVAL2GOBJ(self);
         
-        for (i = 0; i < RARRAY(ary)->len; i++) {
+        for (i = 0; i < RARRAY_LEN(ary); i++) {
             cont_child_set_property(self, other, 
-                       RARRAY(RARRAY(ary)->ptr[i])->ptr[0],
-                       RARRAY(RARRAY(ary)->ptr[i])->ptr[1]);
+                       RARRAY_PTR(RARRAY_PTR(ary)[i])[0],
+                       RARRAY_PTR(RARRAY_PTR(ary)[i])[1]);
         }
     }
     gtk_widget_thaw_child_notify(child);
@@ -360,16 +369,8 @@ cont_get_focus_chain(self)
 {
     gboolean ret;
     GList *glist = NULL;
-    VALUE result;
     ret = gtk_container_get_focus_chain(_SELF(self), &glist);
-
-    if (ret){
-        result = GLIST2ARY(glist);
-        g_list_free(glist);
-    } else {
-        result = Qnil;
-    }
-    return result;
+    return GLIST2ARYF(glist);
 }
 
 static VALUE
@@ -380,8 +381,8 @@ cont_set_focus_chain(self, focusable_widgets)
     GList *glist = NULL;
 
     Check_Type(focusable_widgets, T_ARRAY);
-    for (i = 0; i < RARRAY(focusable_widgets)->len; i++) {
-        glist = g_list_append(glist, RVAL2GOBJ(RARRAY(focusable_widgets)->ptr[i]));
+    for (i = 0; i < RARRAY_LEN(focusable_widgets); i++) {
+        glist = g_list_append(glist, RVAL2GOBJ(RARRAY_PTR(focusable_widgets)[i]));
     }
     gtk_container_set_focus_chain(_SELF(self), glist);
     g_list_free(glist);
@@ -473,7 +474,7 @@ cont_s_child_properties(argc, argv, self)
 
     ary = rb_ary_new();
     for (i = 0; i < n_properties; i++){
-        if (RTEST(inherited_too)
+        if (RVAL2CBOOL(inherited_too)
             || GTYPE2CLASS(props[i]->owner_type) == self)
             rb_ary_push(ary, CSTR2RVAL(props[i]->name));
     }
@@ -711,12 +712,27 @@ type_register(int argc, VALUE* argv, VALUE self)
 }
 /**********************************************************************/
 
+static void
+cont_mark_callback(GtkWidget* w, gpointer data)
+{
+    rbgobj_gc_mark_instance(w);
+}
+
+static void
+cont_mark(void* p)
+{
+    gtk_container_forall(GTK_CONTAINER(p), cont_mark_callback, NULL);
+}
+
+/**********************************************************************/
 
 void 
 Init_gtk_container()
 {
-    VALUE gContainer = G_DEF_CLASS(GTK_TYPE_CONTAINER, "Container", mGtk);
+    VALUE gContainer;
 
+    gContainer = G_DEF_CLASS_WITH_GC_FUNC(GTK_TYPE_CONTAINER, "Container",
+					  mGtk, cont_mark, NULL);
     rb_define_method(gContainer, "resize_container?", cont_is_resize_container, 0);
     rb_define_method(gContainer, "add", cont_add, -1);
     rb_define_alias(gContainer, "<<", "add");
