@@ -3,8 +3,8 @@
  
   rbgtkfilechooser.c -
  
-  $Author: mutoh $
-  $Date: 2006/06/17 06:59:32 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:31 $
  
   Copyright (C) 2005,2006 Ruby-GNOME2 Project Team
   Copyright (C) 2004 Seiya Nishizawa, Masao Mutoh
@@ -16,9 +16,11 @@
 
 #define _SELF(self) GTK_FILE_CHOOSER(RVAL2GOBJ(self))
 
+#ifdef HAVE_GTK_GTKFILESYSTEM_H
 /* For error handling */
-#define GTK_FILE_SYSTEM_ENABLE_UNSUPPORTED
-#include <gtk/gtkfilesystem.h>
+#  define GTK_FILE_SYSTEM_ENABLE_UNSUPPORTED
+#  include <gtk/gtkfilesystem.h>
+#endif
 
 static VALUE
 gslist2ary_free(list)
@@ -51,9 +53,9 @@ fcho_get_action(self)
 }
 
 /*  They are defined as properties
-    gtk_file_chooser_set_local_only(_SELF(self), RTEST(local));
+    gtk_file_chooser_set_local_only(_SELF(self), RVAL2CBOOL(local));
     gtk_file_chooser_get_local_only(_SELF(self));
-    gtk_file_chooser_set_select_multiple(_SELF(self), RTEST(multiple));
+    gtk_file_chooser_set_select_multiple(_SELF(self), RVAL2CBOOL(multiple));
     gtk_file_chooser_get_select_multiple(_SELF(self));
 */
 
@@ -200,9 +202,9 @@ fcho_get_current_folder_uri(self)
 /* They are defined as properties.
     gtk_file_chooser_set_preview_widget(_SELF(self), GTK_WIDGET(RVAL2GOBJ(widget));
     gtk_file_chooser_get_preview_widget(_SELF(self)));
-    gtk_file_chooser_set_preview_widget_active(_SELF(self), RTEST(active));
+    gtk_file_chooser_set_preview_widget_active(_SELF(self), RVAL2CBOOL(active));
     gtk_file_chooser_get_preview_widget_active(_SELF(self));
-    gtk_file_chooser_set_use_preview_label(_SELF(self), RTEST(label));
+    gtk_file_chooser_set_use_preview_label(_SELF(self), RVAL2CBOOL(label));
     gtk_file_chooser_get_use_preview_label(_SELF(self));
 */
 
@@ -324,7 +326,9 @@ Init_gtk_file_chooser()
 #if GTK_CHECK_VERSION(2,4,0)
 
     VALUE gFileCho = G_DEF_INTERFACE(GTK_TYPE_FILE_CHOOSER, "FileChooser", mGtk);
-    VALUE fse;
+#ifdef GTK_FILE_SYSTEM_ERROR
+    VALUE eFileSystemError;
+#endif
 
     rb_define_method(gFileCho, "set_action", fcho_set_action, 1);
     rb_define_method(gFileCho, "action", fcho_get_action, 0);
@@ -367,14 +371,24 @@ Init_gtk_file_chooser()
     /* GtkFileChooserError */
     G_DEF_ERROR(GTK_FILE_CHOOSER_ERROR, "FileChooserError", mGtk, rb_eRuntimeError, 
                 GTK_TYPE_FILE_CHOOSER_ERROR);
+
+#ifdef GTK_FILE_SYSTEM_ERROR
     /* GtkFileSystemError */
-    fse = G_DEF_ERROR2(GTK_FILE_SYSTEM_ERROR, "FileSystemError", mGtk, rb_eRuntimeError);
-    rb_define_const(fse, "NONEXISTENT", INT2NUM(GTK_FILE_SYSTEM_ERROR_NONEXISTENT));
-    rb_define_const(fse, "NOT_FOLDER", INT2NUM(GTK_FILE_SYSTEM_ERROR_NOT_FOLDER));
-    rb_define_const(fse, "INVALID_URI", INT2NUM(GTK_FILE_SYSTEM_ERROR_INVALID_URI));
-    rb_define_const(fse, "BAD_FILENAME", INT2NUM(GTK_FILE_SYSTEM_ERROR_BAD_FILENAME));
-    rb_define_const(fse, "FAILED", INT2NUM(GTK_FILE_SYSTEM_ERROR_FAILED));
-    rb_define_const(fse, "ALREADY_EXSITS", INT2NUM(GTK_FILE_SYSTEM_ERROR_ALREADY_EXISTS));
+    eFileSystemError = G_DEF_ERROR2(GTK_FILE_SYSTEM_ERROR, "FileSystemError",
+				    mGtk, rb_eRuntimeError);
+    rb_define_const(eFileSystemError, "NONEXISTENT",
+		    INT2NUM(GTK_FILE_SYSTEM_ERROR_NONEXISTENT));
+    rb_define_const(eFileSystemError, "NOT_FOLDER",
+		    INT2NUM(GTK_FILE_SYSTEM_ERROR_NOT_FOLDER));
+    rb_define_const(eFileSystemError, "INVALID_URI",
+		    INT2NUM(GTK_FILE_SYSTEM_ERROR_INVALID_URI));
+    rb_define_const(eFileSystemError, "BAD_FILENAME",
+		    INT2NUM(GTK_FILE_SYSTEM_ERROR_BAD_FILENAME));
+    rb_define_const(eFileSystemError, "FAILED",
+		    INT2NUM(GTK_FILE_SYSTEM_ERROR_FAILED));
+    rb_define_const(eFileSystemError, "ALREADY_EXSITS",
+		    INT2NUM(GTK_FILE_SYSTEM_ERROR_ALREADY_EXISTS));
+#endif
 
 #if GTK_CHECK_VERSION(2,8,0)
     /* GtkFileChooserConfirmation */
