@@ -3,8 +3,8 @@
  
   rbgtkfilefilter.c -
  
-  $Author: mutoh $
-  $Date: 2005/01/29 11:44:14 $
+  $Author: sakai $
+  $Date: 2007/07/08 03:00:49 $
  
   Copyright (C) 2005 Ruby-GNOME2 Project Team
   Copyright (C) 2004 Seiya Nishizawa, Masao Mutoh
@@ -81,7 +81,7 @@ static VALUE
 ffil_add_custom(self, needed)
     VALUE self, needed;
 {
-    VALUE func = G_BLOCK_PROC();
+    VALUE func = rb_block_proc();
     G_RELATIVE(self, func);
     gtk_file_filter_add_custom(_SELF(self), RVAL2GFLAGS(needed, GTK_TYPE_FILE_FILTER_FLAGS), 
                                (GtkFileFilterFunc)filter_func, (gpointer)func, NULL);
