@@ -3,8 +3,8 @@
 
   rbgtknotebook.c -
 
-  $Author: mutoh $
-  $Date: 2006/10/21 16:58:00 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:31 $
 
   Copyright (C) 2002,2003 Ruby-GNOME2 Project Team
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
@@ -212,8 +212,8 @@ note_query_tab_label_packing(self, child)
                                          RVAL2WIDGET(child),
                                          &expand, &fill, &pack_type);
     ary = rb_ary_new2(3);
-    rb_ary_push(ary, expand ? Qtrue : Qfalse);
-    rb_ary_push(ary, fill ? Qtrue : Qfalse);
+    rb_ary_push(ary, CBOOL2RVAL(expand));
+    rb_ary_push(ary, CBOOL2RVAL(fill));
     rb_ary_push(ary, GENUM2RVAL(pack_type, GTK_TYPE_PACK_TYPE));
     return ary;
 }
@@ -254,7 +254,7 @@ note_set_tab_label_packing(self, child, expand, fill, pack_type)
 {
     gtk_notebook_set_tab_label_packing(_SELF(self),
                                        RVAL2WIDGET(child),
-                                       RTEST(expand), RTEST(fill),
+                                       RVAL2CBOOL(expand), RVAL2CBOOL(fill),
                                        RVAL2GENUM(pack_type, GTK_TYPE_PACK_TYPE));
     return self;
 }
@@ -336,7 +336,7 @@ static VALUE
 note_set_window_creation_hook(self)
     VALUE self;
 {
-    VALUE func = G_BLOCK_PROC();
+    VALUE func = rb_block_proc();
     G_RELATIVE(self, func);
     gtk_notebook_set_window_creation_hook((GtkNotebookWindowCreationFunc)creation_func, 
                                           (gpointer)func, (GDestroyNotify)NULL);
