@@ -3,8 +3,8 @@
 
   rbgtkselectiondata.c -
 
-  $Author: mutoh $
-  $Date: 2006/11/12 15:28:40 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:32 $
 
   Copyright (C) 2002-2005 Masao Mutoh
 ************************************************/
@@ -18,33 +18,36 @@ static GdkAtom compound_text;
 
 /************************************************************************/
 void
-rbgtk_atom2selectiondata(type, size, src, gtype, data, format, length)
-    VALUE type, size, src;
-    void** data;
-    GdkAtom* gtype;
-    gint* format;
-    gint* length;
+rbgtk_atom2selectiondata(VALUE type, VALUE size, VALUE src, GdkAtom *gtype,
+                         void **data, gint *format, gint *length)
 {
     void* dat = NULL;
     gint fmt, len;
     GdkAtom ntype = RVAL2ATOM(type);
     
     if(ntype == GDK_SELECTION_TYPE_INTEGER){
-        int i = NUM2INT(src);
-        dat = (void*)&i;
-        fmt = 32;
+        int *i;
+        i = ALLOC(int);
+        *i = NUM2INT(src);
+        dat = i;
+        fmt = sizeof(int) * 8;
         len = 1;
     } else if(ntype == GDK_SELECTION_TYPE_STRING) {
         dat = RVAL2CSTR(src);
-        fmt = 8;
-        len = RSTRING(src)->len;
+	if (NIL_P(size)) {
+	    fmt = sizeof(char) * 8;
+	    len = RSTRING_LEN(src);
+	} else {
+	    len = NUM2UINT(size);
+	    fmt = (RSTRING_LEN(src) / len) * 8;
+	}
     } else if(ntype == compound_text){
         guchar* str = (guchar*)dat;
         gdk_string_to_compound_text(RVAL2CSTR(src), &ntype, &fmt, &str, &len);
     } else if(type != Qnil && size != Qnil && src != Qnil) {
     	dat = RVAL2CSTR(src);
 	fmt = NUM2INT(size);
-	len = (RSTRING(src)->len * sizeof(char) / fmt);
+	len = (RSTRING_LEN(src) * sizeof(char) / fmt);
     } else {
         rb_raise(rb_eArgError, "no supported type.");
     }
@@ -60,7 +63,8 @@ rbgtk_atom2selectiondata_free(type, dat)
     GdkAtom type;
     void* dat;
 {
-    if(type == GDK_SELECTION_TYPE_ATOM) {
+    if (type == GDK_SELECTION_TYPE_INTEGER ||
+        type == GDK_SELECTION_TYPE_ATOM) {
         xfree(dat);
     } else if(type == compound_text) {
         gdk_free_compound_text(dat);
@@ -148,7 +152,7 @@ gtkselectiondata_set_text(self, str)
     StringValue(str);
 
     ret = gtk_selection_data_set_text(_SELF(self), RVAL2CSTR(str),
-                                      RSTRING(str)->len);
+                                      RSTRING_LEN(str));
 
     if (!ret) 
         rb_raise(rb_eRuntimeError, "the selection wasn't successfully.");
@@ -198,12 +202,12 @@ gtkselectiondata_set_uris(self, uris)
     int i, n_targets;
     
     Check_Type(uris, T_ARRAY);
-    n_targets = RARRAY(uris)->len;
+    n_targets = RARRAY_LEN(uris);
  
     guris = g_new(gchar*, n_targets + 1);
 
     for (i = 0; i < n_targets; i++) {
-        guris[i] = RVAL2CSTR(RARRAY(uris)->ptr[i]);
+        guris[i] = RVAL2CSTR(RARRAY_PTR(uris)[i]);
     }
     guris[n_targets] = NULL;
 
@@ -260,7 +264,7 @@ static VALUE
 gtkselectiondata_targets_include_image(self, writable)
     VALUE self, writable;
 {
-    return CBOOL2RVAL(gtk_selection_data_targets_include_image(_SELF(self), RTEST(writable)));
+    return CBOOL2RVAL(gtk_selection_data_targets_include_image(_SELF(self), RVAL2CBOOL(writable)));
 }
 #endif
 
