@@ -3,8 +3,8 @@
 
   rbgtksettings.c -
 
-  $Author: mutoh $
-  $Date: 2005/01/29 11:44:15 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:32 $
 
   Copyright (C) 2003,2004 Ruby-GNOME2 Project Team
   Copyright (C) 2003 Masao Mutoh
@@ -48,7 +48,7 @@ rc_property_parser(pspec, rc_string, property_value)
     VALUE ret = rb_funcall(func, id_call, 2, spec, CSTR2RVAL(rc_string->str));
     if (NIL_P(ret)) {
         return FALSE;
-    } else if (RTEST(ret)){
+    } else if (RVAL2CBOOL(ret)){
         rbgobj_rvalue_to_gvalue(ret, property_value);
         return TRUE;
     } else {
@@ -62,7 +62,7 @@ settings_s_install_property(self, spec)
 {
     GParamSpec* pspec = G_PARAM_SPEC(RVAL2GOBJ(spec));
     if (rb_block_given_p()){
-        VALUE func = G_BLOCK_PROC();
+        VALUE func = rb_block_proc();
         rb_hash_aset(prop_func_table, spec, func);
         gtk_settings_install_property_parser(pspec, (GtkRcPropertyParser)rc_property_parser);
     } else {
