@@ -3,8 +3,8 @@
 
   rbgtktextbuffer.c -
 
-  $Author: mutoh $
-  $Date: 2006/11/23 08:39:13 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:32 $
 
   Copyright (C) 2002-2005 Ruby-GNOME2 Project Team
   Copyright (C) 2002,2003 Masahiro Sakai
@@ -66,7 +66,7 @@ txt_set_text(self, text)
     VALUE self, text;
 {
     StringValue(text);
-    gtk_text_buffer_set_text(_SELF(self), RVAL2CSTR(text), RSTRING(text)->len);
+    gtk_text_buffer_set_text(_SELF(self), RVAL2CSTR(text), RSTRING_LEN(text));
     return self;
 }
 
@@ -76,8 +76,8 @@ txt_backspace(self, iter, interactive, default_editable)
     VALUE self, iter, interactive, default_editable;
 {
     return CBOOL2RVAL(gtk_text_buffer_backspace(_SELF(self), RVAL2ITR(iter),
-                                                RTEST(interactive),
-                                                RTEST(default_editable)));
+                                                RVAL2CBOOL(interactive),
+                                                RVAL2CBOOL(default_editable)));
 }
 #endif
 
@@ -86,7 +86,8 @@ txt_insert_at_cursor(self, text)
     VALUE self, text;
 {
     StringValue(text);
-    gtk_text_buffer_insert_at_cursor(_SELF(self), RVAL2CSTR(text), RSTRING(text)->len);
+    gtk_text_buffer_insert_at_cursor(_SELF(self),
+                                     RVAL2CSTR(text), RSTRING_LEN(text));
     return self;
 }
 
@@ -94,24 +95,21 @@ static VALUE
 txt_insert_interactive(self, iter, text, editable)
     VALUE self, iter, text, editable;
 {
-    StringValue(text);
-    return gtk_text_buffer_insert_interactive(_SELF(self), RVAL2ITR(iter),
-                                              RVAL2CSTR(text),
-                                              RSTRING(text)->len,
-                                              RTEST(editable))
-        ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_text_buffer_insert_interactive(_SELF(self),
+                                                         RVAL2ITR(iter),
+                                                         RVAL2CSTR(text),
+                                                         RSTRING_LEN(text),
+                                                         RVAL2CBOOL(editable)));
 }
 
 static VALUE
 txt_insert_interactive_at_cursor(self, text, editable)
     VALUE self, text, editable;
 {
-    StringValue(text);
-    return gtk_text_buffer_insert_interactive_at_cursor(_SELF(self),
-                                                        RVAL2CSTR(text),
-                                                        RSTRING(text)->len,
-                                                        RTEST(editable))
-        ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_text_buffer_insert_interactive_at_cursor(_SELF(self),
+                                                                   RVAL2CSTR(text),
+                                                                   RSTRING_LEN(text),
+                                                                   RVAL2CBOOL(editable)));
 }
 
 static VALUE
@@ -127,12 +125,11 @@ static VALUE
 txt_insert_range_interactive(self, iter, start, end, editable)
     VALUE self, iter, start, end, editable;
 {
-    return gtk_text_buffer_insert_range_interactive(_SELF(self),
-                                                    RVAL2ITR(iter),
-                                                    RVAL2ITR(start),
-                                                    RVAL2ITR(end),
-                                                    RTEST(editable))
-        ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_text_buffer_insert_range_interactive(_SELF(self),
+                                                               RVAL2ITR(iter),
+                                                               RVAL2ITR(start),
+                                                               RVAL2ITR(end),
+                                                               RVAL2CBOOL(editable)));
 }
 
 static VALUE
@@ -148,11 +145,10 @@ static VALUE
 txt_delete_interactive(self, start, end, editable)
     VALUE self, start, end, editable;
 {
-    return gtk_text_buffer_delete_interactive(_SELF(self),
-                                              RVAL2ITR(start),
-                                              RVAL2ITR(end),
-                                              RTEST(editable))
-        ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_text_buffer_delete_interactive(_SELF(self),
+                                                         RVAL2ITR(start),
+                                                         RVAL2ITR(end),
+                                                         RVAL2CBOOL(editable)));
 }
 
 static VALUE
@@ -177,7 +173,7 @@ txt_get_text(argc, argv, self)
             buffer,
             NIL_P(start) ? &start_iter : RVAL2ITR(start),
             NIL_P(end) ? &end_iter : RVAL2ITR(end),
-            RTEST(include_hidden_chars));
+            RVAL2CBOOL(include_hidden_chars));
     result = CSTR2RVAL(ret);
     g_free(ret);
 
@@ -213,7 +209,7 @@ txt_get_slice(argc, argv, self)
             buffer,
             NIL_P(start) ? &start_iter : RVAL2ITR(start),
             NIL_P(end) ? &end_iter : RVAL2ITR(end),
-            RTEST(include_hidden_chars));
+            RVAL2CBOOL(include_hidden_chars));
     result = CSTR2RVAL(ret);
     g_free(ret);
 
@@ -234,7 +230,7 @@ txt_insert_pixbuf(self, iter, pixbuf)
     G_CHILD_ADD(self, iter);
     G_CHILD_ADD(iter, pixbuf);
 
-    if (RTEST(ruby_debug))
+    if (RVAL2CBOOL(ruby_debug))
         rb_warning("Gtk::TextBuffer#insert_pixbuf is deprecated. Use Gtk::TextBuffer#insert instead.");
     gtk_text_buffer_insert_pixbuf(_SELF(self), RVAL2ITR(iter),
                                   GDK_PIXBUF(RVAL2GOBJ(pixbuf)));
@@ -247,7 +243,7 @@ txt_insert_child_anchor(self, iter, anchor)
 {
     G_CHILD_ADD(self, iter);
     G_CHILD_ADD(iter, anchor);
-    if (RTEST(ruby_debug))
+    if (RVAL2CBOOL(ruby_debug))
         rb_warning("Gtk::TextBuffer#insert_child_anchor is deprecated. Use Gtk::TextBuffer#insert instead.");
     gtk_text_buffer_insert_child_anchor(_SELF(self), RVAL2ITR(iter),
                                         GTK_TEXT_CHILD_ANCHOR(RVAL2GOBJ(anchor)));
@@ -270,11 +266,21 @@ txt_create_mark(self, name, where, left_gravity)
     VALUE ret = GOBJ2RVAL(gtk_text_buffer_create_mark(_SELF(self),
                                                       NIL_P(name) ? NULL : RVAL2CSTR(name),
                                                       RVAL2ITR(where),
-                                                      RTEST(left_gravity)));
+                                                      RVAL2CBOOL(left_gravity)));
     G_CHILD_ADD(self, ret);
     return ret;
 }
 
+#if GTK_CHECK_VERSION(2,12,0)
+static VALUE
+txt_add_mark(self, mark, where)
+    VALUE self, mark, where;
+{
+    gtk_text_buffer_add_mark(_SELF(self), RVAL2MARK(mark), RVAL2ITR(where));
+    return self;
+}
+#endif
+
 static VALUE
 txt_delete_mark(self, mark)
     VALUE self, mark;
@@ -343,14 +349,14 @@ static VALUE
 txt_get_modified(self)
     VALUE self;
 {
-    return gtk_text_buffer_get_modified(_SELF(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_text_buffer_get_modified(_SELF(self)));
 }
 
 static VALUE
 txt_set_modified(self, setting)
     VALUE self, setting;
 {
-    gtk_text_buffer_set_modified(_SELF(self), RTEST(setting));
+    gtk_text_buffer_set_modified(_SELF(self), RVAL2CBOOL(setting));
     return setting;
 }
 
@@ -384,8 +390,8 @@ txt_deserialize(self, content_buffer, format, iter, data)
     ret = gtk_text_buffer_deserialize(_SELF(self), _SELF(content_buffer),
                                       RVAL2ATOM(format),
                                       RVAL2ITR(iter),
-                                      (const guint8*)RSTRING(data)->ptr,
-                                      (gsize)RSTRING(data)->len,
+                                      (const guint8*)RSTRING_PTR(data),
+                                      (gsize)RSTRING_LEN(data),
                                       &error);
     if (! ret) RAISE_GERROR(error);
     return self;
@@ -406,7 +412,7 @@ txt_deserialize_set_can_create_tags(self, format, can_create_tags)
 {
     gtk_text_buffer_deserialize_set_can_create_tags(_SELF(self),
                                                     RVAL2ATOM(format),
-                                                    RTEST(can_create_tags));
+                                                    RVAL2CBOOL(can_create_tags));
     return self;
 }
 
@@ -491,7 +497,7 @@ deserialize_func(register_buffer, content_buffer, iter, data, length, create_tag
     arg.argv = argv;
 
     result = G_PROTECT_CALLBACK(invoke_callback, &arg);
-    return NIL_P(ruby_errinfo) ? RVAL2CBOOL(result) : FALSE;
+    return NIL_P(rb_errinfo()) ? RVAL2CBOOL(result) : FALSE;
 }
 
 static void
@@ -504,7 +510,7 @@ static VALUE
 txt_register_deserialize_format(self, mime_type)
     VALUE self, mime_type;
 {
-    VALUE block = G_BLOCK_PROC();
+    VALUE block = rb_block_proc();
     GdkAtom atom;
     G_CHILD_ADD(mGtk, block);
     atom = gtk_text_buffer_register_deserialize_format(_SELF(self),
@@ -548,15 +554,15 @@ serialize_func(register_buffer, content_buffer, start, end, length, func)
     /* This should return data as String */
     result = G_PROTECT_CALLBACK(invoke_callback, &arg);
     Check_Type(result, T_STRING);
-    *length = RSTRING(result)->len;
-    return (guint8*)(NIL_P(ruby_errinfo) ? RSTRING(result)->ptr : NULL);
+    *length = RSTRING_LEN(result);
+    return (guint8*)(NIL_P(rb_errinfo()) ? RSTRING_PTR(result) : NULL);
 }
 
 static VALUE
 txt_register_serialize_format(self, mime_type)
     VALUE self, mime_type;
 {
-    VALUE block = G_BLOCK_PROC();
+    VALUE block = rb_block_proc();
     GdkAtom atom;
     G_CHILD_ADD(mGtk, block);
     atom = gtk_text_buffer_register_serialize_format(_SELF(self),
@@ -609,7 +615,7 @@ txt_cut_clipboard(self, clipboard, default_editable)
     VALUE self, clipboard, default_editable;
 {
     G_CHILD_ADD(self, clipboard);
-    gtk_text_buffer_cut_clipboard(_SELF(self), RVAL2CLIPBOARD(clipboard), RTEST(default_editable));
+    gtk_text_buffer_cut_clipboard(_SELF(self), RVAL2CLIPBOARD(clipboard), RVAL2CBOOL(default_editable));
     return self;
 }
 
@@ -629,7 +635,7 @@ txt_paste_clipboard(self, clipboard, location, default_editable)
     G_CHILD_ADD(self, clipboard);
     gtk_text_buffer_paste_clipboard(_SELF(self), RVAL2CLIPBOARD(clipboard),
                                     NIL_P(location) ? NULL : RVAL2ITR(location),
-                                    RTEST(default_editable));
+                                    RVAL2CBOOL(default_editable));
     return self;
 }
 
@@ -656,8 +662,8 @@ txt_delete_selection(argc, argv, self)
 {
     VALUE interactive, default_editable;
     rb_scan_args(argc, argv, "20", &interactive, &default_editable); 
-    return gtk_text_buffer_delete_selection(_SELF(self),
-        RTEST(interactive), RTEST(default_editable)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_text_buffer_delete_selection(_SELF(self),
+                                                       RVAL2CBOOL(interactive), RVAL2CBOOL(default_editable)));
 }
 
 static VALUE
@@ -733,13 +739,11 @@ txt_insert(argc, argv, self)
     VALUE self;
 {
     VALUE where, value, tags;
-    struct RArray *tarray;
     gint start_offset;
     GtkTextIter start;
     int i;
 
     rb_scan_args(argc, argv, "2*", &where, &value, &tags);
-    tarray = RARRAY(tags);
 
     G_CHILD_ADD(self, where);
     G_CHILD_ADD(self, value);
@@ -752,24 +756,27 @@ txt_insert(argc, argv, self)
     } else {
         start_offset = gtk_text_iter_get_offset(RVAL2ITR(where));
         StringValue(value);
-        gtk_text_buffer_insert(_SELF(self), RVAL2ITR(where), RVAL2CSTR(value), RSTRING(value)->len);
+        gtk_text_buffer_insert(_SELF(self), RVAL2ITR(where),
+                               RVAL2CSTR(value), RSTRING_LEN(value));
         
-        if(tarray->len == 0)
+        if (RARRAY_LEN(tags) == 0)
             return self;
 
         G_CHILD_ADD(self, tags);
         
         gtk_text_buffer_get_iter_at_offset(_SELF(self), &start, start_offset);
         
-        for(i=0; i<tarray->len; i++) {
+        for(i = 0; i < RARRAY_LEN(tags); i++) {
             GtkTextTag *tag;
             
-            if (rb_obj_is_kind_of(tarray->ptr[i], GTYPE2CLASS(GTK_TYPE_TEXT_TAG))) {
-                tag = RVAL2GOBJ(tarray->ptr[i]);
+            if (rb_obj_is_kind_of(RARRAY_PTR(tags)[i], GTYPE2CLASS(GTK_TYPE_TEXT_TAG))) {
+                tag = RVAL2GOBJ(RARRAY_PTR(tags)[i]);
             } else {
-                tag = gtk_text_tag_table_lookup(_SELF(self)->tag_table, RVAL2CSTR(tarray->ptr[i]));
+                tag = gtk_text_tag_table_lookup(_SELF(self)->tag_table,
+						RVAL2CSTR(RARRAY_PTR(tags)[i]));
                 if (tag == NULL) {
-                    g_warning ("%s: no tag with name '%s'!", G_STRLOC, RVAL2CSTR(tarray->ptr[i]));
+                    g_warning ("%s: no tag with name '%s'!",
+			       G_STRLOC, RVAL2CSTR(RARRAY_PTR(tags)[i]));
                     return self;
                 }
             }
@@ -785,7 +792,7 @@ txt_insert_with_tags(argc, argv, self)
     VALUE *argv;
     VALUE self;
 {
-    if (RTEST(ruby_debug))
+    if (RVAL2CBOOL(ruby_debug))
         rb_warning("Gtk::TextBuffer#insert_with_tags is deprecated. Use Gtk::TextBuffer#insert instead.");
     txt_insert(argc, argv, self);
     return self;
@@ -921,6 +928,9 @@ Init_gtk_textbuffer()
     rb_define_method(gTextBuffer, "create_child_anchor", txt_create_child_anchor, 1);
     
     rb_define_method(gTextBuffer, "create_mark", txt_create_mark, 3);
+#if GTK_CHECK_VERSION(2,12,0)
+    rb_define_method(gTextBuffer, "add_mark", txt_add_mark, 2);
+#endif
     rb_define_method(gTextBuffer, "delete_mark", txt_delete_mark, 1);
 
     rb_define_method(gTextBuffer, "get_mark", txt_get_mark, 1);
