@@ -1,13 +1,13 @@
-/* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
+/* -*- c-file-style: "ruby" -*- */
 /************************************************
 
   rbglade.c -
 
-  $Author: ktou $
-  $Date: 2006/11/08 00:57:15 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:33 $
 
 
-  Copyright (C) 2002-2005 Ruby-GNOME2 Project
+  Copyright (C) 2002-2009 Ruby-GNOME2 Project
 
   This program is free software.
   You can distribute/modify this program under the terms of
@@ -33,6 +33,21 @@ static const int RB_GLADE_XML_BUFFER = 2;
 
 static VALUE cGladeXML;
 
+static void
+rb_gladexml_mark(gpointer pointer)
+{
+    GladeXML *xml = pointer;
+    GList *names, *node;
+
+    names = glade_xml_get_widget_prefix(xml, "");
+    for (node = names; node; node = g_list_next(node)) {
+	GtkWidget *widget = node->data;
+
+	rbgobj_gc_mark_instance(widget);
+    }
+    g_list_free(names);
+}
+
 static VALUE
 rb_gladexml_get_tooltips(VALUE self, VALUE toplevel)
 {
@@ -89,7 +104,7 @@ xml_autoconnect(const gchar *handler_name, GObject *_source,
 static VALUE
 rb_gladexml_signal_autoconnect_full(VALUE self)
 {
-    rb_iv_set(self, "@autoconnect_proc", G_BLOCK_PROC());
+    rb_iv_set(self, "@autoconnect_proc", rb_block_proc());
     glade_xml_signal_autoconnect_full(GLADE_XML(RVAL2GOBJ(self)), 
                                       xml_autoconnect, (gpointer)self);
     return self;
@@ -160,15 +175,17 @@ rb_gladexml_initialize(int argc, VALUE *argv, VALUE self)
         xml = glade_xml_new(RVAL2CSTR(text), root, domain);
     } else if (dflag == RB_GLADE_XML_BUFFER) {
         StringValue(text);
-        xml = glade_xml_new_from_buffer(RVAL2CSTR(text), RSTRING(text)->len, root, domain);
+        xml = glade_xml_new_from_buffer(RVAL2CSTR(text), RSTRING_LEN(text),
+                                        root, domain);
+        rb_iv_set(self, "@xml", text);
     } else {
-         rb_raise(rb_eArgError, "flag is wrong valiable %d", dflag);
+        rb_raise(rb_eArgError, "flag is wrong variable %d", dflag);
     }
 
     if (xml) {
         G_INITIALIZE(self, xml);
         if (rb_block_given_p()){
-            rb_iv_set(self, "@handler_proc", G_BLOCK_PROC());
+            rb_iv_set(self, "@handler_proc", rb_block_proc());
         } else {
             rb_iv_set(self, "@handler_proc", Qnil);
         }
@@ -177,7 +194,6 @@ rb_gladexml_initialize(int argc, VALUE *argv, VALUE self)
         rb_raise(rb_eIOError, "could not load glade file %s", RVAL2CSTR(text));
     }
 
-    rb_funcall(self, rb_intern("guard_sources_from_gc"), 0);
     return self;
 }
 
@@ -242,7 +258,7 @@ static gboolean custom_widget_supported = FALSE;
 static VALUE
 rb_gladexml_set_custom_widget_handler(VALUE self, VALUE setting)
 {
-    if (RTEST(setting)){
+    if (RVAL2CBOOL(setting)){
         glade_set_custom_handler(custom_widget_handler, NULL);
         custom_widget_supported = TRUE;
     } else {
@@ -269,7 +285,8 @@ rb_gladexml_provide(VALUE self, VALUE library)
 void 
 Init_libglade2()
 {
-    cGladeXML = G_DEF_CLASS(GLADE_TYPE_XML, "GladeXML", rb_cObject);
+    cGladeXML = G_DEF_CLASS_WITH_GC_FUNC(GLADE_TYPE_XML, "GladeXML", rb_cObject,
+					 rb_gladexml_mark, NULL);
 
     rb_define_method(cGladeXML, "signal_autoconnect_full", rb_gladexml_signal_autoconnect_full, 0);
     rb_define_method(cGladeXML, "initialize", rb_gladexml_initialize, -1);
