@@ -8,7 +8,12 @@
 
 require 'glib2'
 begin
-  require 'cairo'
+  begin
+    require 'cairo'
+  rescue LoadError
+    require 'rubygems'
+    require 'cairo'
+  end
 rescue LoadError
 end
 require 'pango.so'
