@@ -3,6 +3,7 @@ extconf.rb for Ruby/RSVG extention library
 =end
 
 PACKAGE_NAME = "rsvg2"
+PACKAGE_ID   = "librsvg-2.0"
 
 TOPDIR = File.expand_path(File.dirname(__FILE__) + '/..')
 MKMF_GNOME2_DIR = TOPDIR + '/glib/src/lib'
@@ -12,7 +13,7 @@ $LOAD_PATH.unshift MKMF_GNOME2_DIR
 
 require 'mkmf-gnome2'
 
-PKGConfig.have_package('librsvg-2.0') or exit 1
+PKGConfig.have_package(PACKAGE_ID) or exit 1
 setup_win32(PACKAGE_NAME)
 
 rsvg_header = "librsvg/rsvg.h"
@@ -30,6 +31,8 @@ check_cairo and have_header("librsvg/rsvg-cairo.h")
 
 add_depend_package("glib2", "glib/src", TOPDIR)
 
+create_pkg_config_file("Ruby/RSVG", PACKAGE_ID, nil, "ruby-rsvg2.pc")
+
 create_makefile_at_srcdir(PACKAGE_NAME, SRCDIR, "-DRUBY_RSVG2_COMPILATION") do
   enum_type_prefix = "librsvg-enum-types"
   unless have_header("librsvg/#{enum_type_prefix}.h")
