@@ -22,7 +22,13 @@ add_depend_package("glib2", "glib/src", TOPDIR)
 add_depend_package("gtk2", "gtk/src", TOPDIR)
 add_depend_package("atk", "atk/src", TOPDIR)
 
-make_version_header("VTE", PACKAGE_ID)
+vte_headers = ["vte/vte.h"]
+have_type("VteTerminalCursorBlinkMode", vte_headers)
 
+unless have_macro("VTE_CHECK_VERSION", vte_headers)
+  make_version_header("VTE", PACKAGE_ID)
+end
+
+create_pkg_config_file("Ruby/VTE", PACKAGE_ID)
 create_makefile_at_srcdir(PACKAGE_NAME, SRCDIR, "-DRUBY_VTE_COMPILATION")
 create_top_makefile
