@@ -405,10 +405,71 @@ term_set_background_transparent(VALUE self, VALUE transparent)
 static VALUE
 term_set_cursor_blinks(VALUE self, VALUE blink)
 {
+#if VTE_CHECK_VERSION(0, 18, 0)
+    VteTerminalCursorBlinkMode mode;
+
+    mode = RVAL2CBOOL(blink) ? VTE_CURSOR_BLINK_ON : VTE_CURSOR_BLINK_OFF;
+    vte_terminal_set_cursor_blink_mode(RVAL2TERM(self), mode);
+#else
     vte_terminal_set_cursor_blinks(RVAL2TERM(self), RVAL2CBOOL(blink));
+#endif
     return Qnil;
 }
 
+#if VTE_CHECK_VERSION(0, 18, 0)
+static VALUE
+term_set_cursor_blink_mode(VALUE self, VALUE rb_mode)
+{
+    VteTerminalCursorBlinkMode mode;
+
+    mode = RVAL2GENUM(rb_mode, VTE_TYPE_TERMINAL_CURSOR_BLINK_MODE);
+    vte_terminal_set_cursor_blink_mode(RVAL2TERM(self), mode);
+    return Qnil;
+}
+
+static VALUE
+term_get_cursor_blink_mode(VALUE self)
+{
+    VteTerminalCursorBlinkMode mode;
+
+    mode = vte_terminal_get_cursor_blink_mode(RVAL2TERM(self));
+    return GENUM2RVAL(mode, VTE_TYPE_TERMINAL_CURSOR_BLINK_MODE);
+}
+#endif
+
+#if VTE_CHECK_VERSION(0, 19, 1)
+static VALUE
+term_set_cursor_shape(VALUE self, VALUE rb_shape)
+{
+    VteTerminalCursorShape shape;
+
+    shape = RVAL2GENUM(rb_shape, VTE_TYPE_TERMINAL_CURSOR_SHAPE);
+    vte_terminal_set_cursor_shape(RVAL2TERM(self), shape);
+    return Qnil;
+}
+
+static VALUE
+term_get_cursor_shape(VALUE self)
+{
+    VteTerminalCursorShape shape;
+
+    shape = vte_terminal_get_cursor_shape(RVAL2TERM(self));
+    return GENUM2RVAL(shape, VTE_TYPE_TERMINAL_CURSOR_SHAPE);
+}
+
+static VALUE
+term_get_pty(VALUE self)
+{
+    return INT2NUM(vte_terminal_get_pty(RVAL2TERM(self)));
+}
+
+static VALUE
+term_get_child_exit_status(VALUE self)
+{
+    return INT2NUM(vte_terminal_get_child_exit_status(RVAL2TERM(self)));
+}
+#endif
+
 static VALUE
 term_set_scrollback_lines(VALUE self, VALUE lines)
 {
@@ -796,6 +857,12 @@ void
 Init_vte_terminal(VALUE mVte)
 {
     VALUE cTerminal, cTerminalEraseBinding, cTerminalAntiAlias;
+#if VTE_CHECK_VERSION(0, 18, 0)
+    VALUE cTerminalCursorBlinkMode;
+#endif
+#if VTE_CHECK_VERSION(0, 19, 1)
+    VALUE cTerminalCursorShape;
+#endif
 
     id_new = rb_intern("new");
     id_call = rb_intern("call");
@@ -810,6 +877,14 @@ Init_vte_terminal(VALUE mVte)
     cTerminal = G_DEF_CLASS(VTE_TYPE_TERMINAL, "Terminal", mVte);
     cTerminalEraseBinding = G_DEF_CLASS(VTE_TYPE_TERMINAL_ERASE_BINDING,
                                         "TerminalEraseBinding", mVte);
+#if VTE_CHECK_VERSION(0, 18, 0)
+    cTerminalCursorBlinkMode = G_DEF_CLASS(VTE_TYPE_TERMINAL_CURSOR_BLINK_MODE,
+					   "TerminalCursorBlinkMode", mVte);
+#endif
+#if VTE_CHECK_VERSION(0, 19, 1)
+    cTerminalCursorShape = G_DEF_CLASS(VTE_TYPE_TERMINAL_CURSOR_SHAPE,
+				       "TerminalCursorShape", mVte);
+#endif
     cTerminalAntiAlias = G_DEF_CLASS(VTE_TYPE_TERMINAL_ANTI_ALIAS,
                                      "TerminalAntiAlias", mVte);
 
@@ -876,6 +951,19 @@ Init_vte_terminal(VALUE mVte)
     rb_define_method(cTerminal, "set_background_transparent",
                      term_set_background_transparent, 1);
     rb_define_method(cTerminal, "set_cursor_blinks", term_set_cursor_blinks, 1);
+#if VTE_CHECK_VERSION(0, 18, 0)
+    rb_define_method(cTerminal, "set_cursor_blink_mode",
+		     term_set_cursor_blink_mode, 1);
+    rb_define_method(cTerminal, "cursor_blink_mode",
+		     term_get_cursor_blink_mode, 0);
+#endif
+#if VTE_CHECK_VERSION(0, 19, 1)
+    rb_define_method(cTerminal, "set_cursor_shape", term_set_cursor_shape, 1);
+    rb_define_method(cTerminal, "cursor_shape", term_get_cursor_shape, 0);
+    rb_define_method(cTerminal, "pty", term_get_pty, 0);
+    rb_define_method(cTerminal, "child_exit_status",
+		     term_get_child_exit_status, 0);
+#endif
     rb_define_method(cTerminal, "set_scrollback_lines",
                      term_set_scrollback_lines, 1);
 
